@@ -23,7 +23,7 @@ package org.apache.catalina;
  * Global constants that are applicable to multiple packages within Catalina.
  *
  * @author Craig R. McClanahan
- * @version $Id: Globals.java 939350 2010-04-29 15:36:29Z kkolinko $
+ * @version $Id: Globals.java 1200601 2011-11-10 22:24:17Z kkolinko $
  */
 
 public final class Globals {
@@ -326,6 +326,17 @@ public final class Globals {
         "javax.servlet.context.tempdir";
 
 
+    /**
+     * The request attribute that is set to <code>Boolean.TRUE</code> if some request
+     * parameters have been ignored during request parameters parsing. It can
+     * happen, for example, if there is a limit on the total count of parseable
+     * parameters, or if parameter cannot be decoded, or any other error
+     * happened during parameter parsing.
+     */
+    public static final String PARAMETER_PARSE_FAILED_ATTR =
+        "org.apache.catalina.parameter_parse_failed";
+
+
     /**
      * The master flag which controls strict servlet specification 
      * compliance.
