@@ -14,10 +14,9 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina;
 
+import java.util.regex.Pattern;
 
 
 /**
@@ -41,7 +40,7 @@ package org.apache.catalina;
  * of Context (representing an individual servlet context).
  *
  * @author Craig R. McClanahan
- * @version $Id: Host.java 939350 2010-04-29 15:36:29Z kkolinko $
+ * @version $Id: Host.java 1060467 2011-01-18 17:01:07Z markt $
  */
 
 public interface Host extends Container {
@@ -181,6 +180,30 @@ public interface Host extends Container {
     public void setXmlNamespaceAware(boolean xmlNamespaceAware);
 
 
+    /**
+     * Return the regular expression that defines the files and directories in
+     * the host's {@link #appBase} that will be ignored by the automatic
+     * deployment process.
+     */
+    public String getDeployIgnore();
+
+
+    /**
+     * Return the compiled regular expression that defines the files and
+     * directories in the host's {@link #appBase} that will be ignored by the
+     * automatic deployment process.
+     */
+    public Pattern getDeployIgnorePattern();
+
+
+    /**
+     * Set the regular expression that defines the files and directories in
+     * the host's {@link #appBase} that will be ignored by the automatic
+     * deployment process.
+     */
+    public void setDeployIgnore(String deployIgnore);
+
+
     // --------------------------------------------------------- Public Methods
 
 
