@@ -31,7 +31,7 @@ import javax.servlet.http.HttpSession;
  * between requests for a particular user of a web application.
  *
  * @author Craig R. McClanahan
- * @version $Id: Session.java 939350 2010-04-29 15:36:29Z kkolinko $
+ * @version $Id: Session.java 1135497 2011-06-14 11:41:45Z markt $
  */
 
 public interface Session {
@@ -111,13 +111,25 @@ public interface Session {
 
 
     /**
-     * Set the session identifier for this session.
+     * Set the session identifier for this session and notifies any associated
+     * listeners that a new session has been created.
      *
      * @param id The new session identifier
      */
     public void setId(String id);
 
 
+    /**
+     * Set the session identifier for this session and optionally notifies any
+     * associated listeners that a new session has been created.
+     *
+     * @param id        The new session identifier
+     * @param notify    Should any associated listeners be notified that a new
+     *                      session has been created? 
+     */
+    public void setId(String id, boolean notify);
+
+
     /**
      * Return descriptive information about this Session implementation and
      * the corresponding version number, in the format
