@@ -18,7 +18,6 @@
 
 package org.apache.catalina.connector;
 
-import java.net.URLEncoder;
 import java.util.HashMap;
 
 import javax.management.MBeanRegistration;
@@ -49,7 +48,7 @@ import org.apache.tomcat.util.modeler.Registry;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: Connector.java 939336 2010-04-29 15:00:41Z kkolinko $
+ * @version $Id: Connector.java 1200601 2011-11-10 22:24:17Z kkolinko $
  */
 
 
@@ -192,6 +191,13 @@ public class Connector
         StringManager.getManager(Constants.Package);
 
 
+    /**
+     * The maximum number of parameters (GET plus POST) which will be
+     * automatically parsed by the container. 10000 by default. A value of less
+     * than 0 means no limit.
+     */
+    protected int maxParameterCount = 10000;
+
     /**
      * Maximum size of a POST which will be automatically parsed by the
      * container. 2MB by default.
@@ -503,14 +509,34 @@ public class Connector
     }
 
 
-     /**
-      * Return the mapper.
-      */
-     public Mapper getMapper() {
+    /**
+     * Return the mapper.
+     */
+    public Mapper getMapper() {
+        return (mapper);
+    }
 
-         return (mapper);
 
-     }
+    /**
+     * Return the maximum number of parameters (GET plus POST) that will be
+     * automatically parsed by the container. A value of less than 0 means no
+     * limit.
+     */
+    public int getMaxParameterCount() {
+        return maxParameterCount;
+    }
+
+
+    /**
+     * Set the maximum number of parameters (GET plus POST) that will be
+     * automatically parsed by the container. A value of less than 0 means no
+     * limit.
+     *
+     * @param maxParameterCount The new setting
+     */
+    public void setMaxParameterCount(int maxParameterCount) {
+        this.maxParameterCount = maxParameterCount;
+    }
 
 
     /**
@@ -887,6 +913,15 @@ public class Connector
         return useIPVHosts;
     }
 
+
+    public String getExecutorName() {
+        Object ex = IntrospectionUtils.getProperty(protocolHandler, "executor");
+        if (ex instanceof org.apache.catalina.Executor) {
+            return ((org.apache.catalina.Executor) ex).getName();
+        }
+        return "Internal";
+    }
+
     // --------------------------------------------------------- Public Methods
 
 
@@ -1259,4 +1294,18 @@ public class Connector
         getService().removeConnector(this);
     }
 
+    /**
+     * Provide a useful toString() implementation as it may be used when logging
+     * Lifecycle errors to identify the component.
+     */
+    @Override
+    public String toString() {
+        // Not worth caching this right now
+        StringBuilder sb = new StringBuilder("Connector[");
+        sb.append(getProtocol());
+        sb.append('-');
+        sb.append(getPort());
+        sb.append(']');
+        return sb.toString();
+    }
 }
