@@ -42,7 +42,6 @@ import javax.servlet.ServletContextAttributeListener;
 import org.apache.catalina.Context;
 import org.apache.catalina.Host;
 import org.apache.catalina.Wrapper;
-import org.apache.catalina.deploy.ApplicationParameter;
 import org.apache.catalina.util.Enumerator;
 import org.apache.catalina.util.RequestUtil;
 import org.apache.catalina.util.ResourceSet;
@@ -63,7 +62,7 @@ import org.apache.catalina.Globals;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: ApplicationContext.java 946844 2010-05-21 01:19:48Z kkolinko $
+ * @version $Id: ApplicationContext.java 1022315 2010-10-13 22:41:45Z kkolinko $
  */
 
 public class ApplicationContext
@@ -122,7 +121,8 @@ public class ApplicationContext
     /**
      * The merged context initialization parameters for this Context.
      */
-    private Map parameters = null;
+    private Map<String,String> parameters =
+        new ConcurrentHashMap<String,String>();
 
 
     /**
@@ -251,10 +251,17 @@ public class ApplicationContext
      * @param name Name of the initialization parameter to retrieve
      */
     public String getInitParameter(final String name) {
+        return parameters.get(name);
+    }
 
-        mergeParameters();
-        return ((String) parameters.get(name));
 
+    public boolean setInitParameter(String name, String value) {
+        if (parameters.containsKey(name)) {
+            return false;
+        }
+        
+        parameters.put(name, value);
+        return true;
     }
 
 
@@ -263,10 +270,7 @@ public class ApplicationContext
      * empty enumeration if the context has no initialization parameters.
      */
     public Enumeration getInitParameterNames() {
-
-        mergeParameters();
         return (new Enumerator(parameters.keySet()));
-
     }
 
 
@@ -867,38 +871,6 @@ public class ApplicationContext
     }
 
 
-    // -------------------------------------------------------- Private Methods
-
-
-    /**
-     * Merge the context initialization parameters specified in the application
-     * deployment descriptor with the application parameters described in the
-     * server configuration, respecting the <code>override</code> property of
-     * the application parameters appropriately.
-     */
-    private void mergeParameters() {
-
-        if (parameters != null)
-            return;
-        Map results = new ConcurrentHashMap();
-        String names[] = context.findParameters();
-        for (int i = 0; i < names.length; i++)
-            results.put(names[i], context.findParameter(names[i]));
-        ApplicationParameter params[] =
-            context.findApplicationParameters();
-        for (int i = 0; i < params.length; i++) {
-            if (params[i].getOverride()) {
-                if (results.get(params[i].getName()) == null)
-                    results.put(params[i].getName(), params[i].getValue());
-            } else {
-                results.put(params[i].getName(), params[i].getValue());
-            }
-        }
-        parameters = results;
-
-    }
-
-
     /**
      * List resource paths (recursively), and store all of them in the given
      * Set.
