@@ -23,6 +23,8 @@ import java.lang.reflect.Method;
 import java.net.MalformedURLException;
 import java.net.URL;
 import java.net.URLConnection;
+import java.sql.DriverManager;
+import java.util.StringTokenizer;
 
 import javax.imageio.ImageIO;
 import javax.xml.parsers.DocumentBuilderFactory;
@@ -66,6 +68,18 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
     public void setAppContextProtection(boolean appContextProtection) {
         this.appContextProtection = appContextProtection;
     }
+
+    /**
+     * Protect against the memory leak caused when the first call to
+     * <code>java.awt.Toolkit.getDefaultToolkit()</code> is triggered
+     * by a web application. Defaults to <code>false</code> because a new
+     * Thread is launched.
+     */
+    private boolean awtThreadProtection = false;
+    public boolean isAWTThreadProtection() { return awtThreadProtection; }
+    public void setAWTThreadProtection(boolean awtThreadProtection) {
+      this.awtThreadProtection = awtThreadProtection;
+    }
     
     /**
      * Protect against the memory leak caused when the first call to
@@ -150,8 +164,8 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
     }
     
     /**
-     * <code>com.sun.jndi.ldap.LdapPoolManager</code> class spawns a thread when it
-     * is initialized if the system property
+     * <code>com.sun.jndi.ldap.LdapPoolManager</code> class spawns a thread when
+     * it is initialized if the system property
      * <code>com.sun.jndi.ldap.connect.pool.timeout</code> is greater than 0.
      * That thread inherits the context class loader of the current thread, so
      * that there may be a web application class loader leak if the web app
@@ -162,6 +176,33 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
     public void setLdapPoolProtection(boolean ldapPoolProtection) {
         this.ldapPoolProtection = ldapPoolProtection;
     }
+
+    /**
+     * The first access to {@link DriverManager} will trigger the loading of
+     * all {@link java.sql.Driver}s in the the current class loader. The web
+     * application level memory leak protection can take care of this in most
+     * cases but triggering the loading here has fewer side-effects. 
+     */
+    private boolean driverManagerProtection = true;
+    public boolean isDriverManagerProtection() {
+        return driverManagerProtection;
+    }
+    public void setDriverManagerProtection(boolean driverManagerProtection) {
+        this.driverManagerProtection = driverManagerProtection;
+    }
+
+    /**
+     * List of comma-separated fully qualified class names to load and initialize during
+     * the startup of this Listener. This allows to pre-load classes that are known to 
+     * provoke classloader leaks if they are loaded during a request processing.
+     */
+    private String classesToInitialize = null;
+    public String getClassesToInitialize() {
+        return classesToInitialize;
+    }
+    public void setClassesToInitialize(String classesToInitialize) {
+        this.classesToInitialize = classesToInitialize;
+    }
     
     public void lifecycleEvent(LifecycleEvent event) {
         // Initialise these classes when Tomcat starts
@@ -176,6 +217,14 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                 Thread.currentThread().setContextClassLoader(
                         ClassLoader.getSystemClassLoader());
 
+                /*
+                 * First call to this loads all drivers in the current class
+                 * loader
+                 */
+                if (driverManagerProtection) {
+                    DriverManager.getDrivers();
+                }
+
                 /*
                  * Several components end up calling:
                  * sun.awt.AppContext.getAppContext()
@@ -194,6 +243,12 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                 if (appContextProtection) {
                     ImageIO.getCacheDirectory();
                 }
+
+                // Trigger the creation of the AWT (AWT-Windows, AWT-XAWT,
+                // etc.) thread
+                if(awtThreadProtection) {
+                  java.awt.Toolkit.getDefaultToolkit();
+                }
                 
                 /*
                  * Several components end up calling:
@@ -352,6 +407,22 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                         }
                     }
                 }
+                
+                if (classesToInitialize != null) {
+                    StringTokenizer strTok =
+                        new StringTokenizer(classesToInitialize, ", \r\n\t");
+                    while (strTok.hasMoreTokens()) {
+                        String classNameToLoad = strTok.nextToken();
+                        try {
+                            Class.forName(classNameToLoad);
+                        } catch (ClassNotFoundException e) {
+                            log.error(
+                                sm.getString("jreLeakListener.classToInitializeFail",
+                                    classNameToLoad), e);
+                            // continue with next class to load
+                        }
+                    }
+                }
 
             } finally {
                 Thread.currentThread().setContextClassLoader(loader);
