@@ -266,7 +266,9 @@ public class StandardService
                 try {
                     connector.initialize();
                 } catch (LifecycleException e) {
-                    log.error("Connector.initialize", e);
+                    log.error(sm.getString(
+                            "standardService.connector.initFailed",
+                            connector), e);
                 }
             }
 
@@ -274,7 +276,9 @@ public class StandardService
                 try {
                     ((Lifecycle) connector).start();
                 } catch (LifecycleException e) {
-                    log.error("Connector.start", e);
+                    log.error(sm.getString(
+                            "standardService.connector.startFailed",
+                            connector), e);
                 }
             }
 
@@ -338,7 +342,9 @@ public class StandardService
                 try {
                     ((Lifecycle) connectors[j]).stop();
                 } catch (LifecycleException e) {
-                    log.error("Connector.stop", e);
+                    log.error(sm.getString(
+                            "standardService.connector.stopFailed",
+                            connectors[j]), e);
                 }
             }
             connectors[j].setContainer(null);
@@ -530,8 +536,13 @@ public class StandardService
         // Start our defined Connectors second
         synchronized (connectors) {
             for (int i = 0; i < connectors.length; i++) {
-                if (connectors[i] instanceof Lifecycle)
+                try {
                     ((Lifecycle) connectors[i]).start();
+                } catch (Exception e) {
+                    log.error(sm.getString(
+                            "standardService.connector.startFailed",
+                            connectors[i]), e);
+                }
             }
         }
         
@@ -563,7 +574,13 @@ public class StandardService
         // Stop our defined Connectors first
         synchronized (connectors) {
             for (int i = 0; i < connectors.length; i++) {
-                connectors[i].pause();
+                try {
+                    connectors[i].pause();
+                } catch (Exception e) {
+                    log.error(sm.getString(
+                            "standardService.connector.pauseFailed",
+                            connectors[i]), e);
+                }
             }
         }
 
@@ -592,8 +609,13 @@ public class StandardService
         // Stop our defined Connectors first
         synchronized (connectors) {
             for (int i = 0; i < connectors.length; i++) {
-                if (connectors[i] instanceof Lifecycle)
+                try {
                     ((Lifecycle) connectors[i]).stop();
+                } catch (Exception e) {
+                    log.error(sm.getString(
+                            "standardService.connector.stopFailed",
+                            connectors[i]), e);
+                }
             }
         }
 
@@ -676,9 +698,19 @@ public class StandardService
 
         // Initialize our defined Connectors
         synchronized (connectors) {
-                for (int i = 0; i < connectors.length; i++) {
+            for (int i = 0; i < connectors.length; i++) {
+                try {
                     connectors[i].initialize();
+                } catch (Exception e) {
+                    String message = sm.getString(
+                            "standardService.connector.initFailed",
+                            connectors[i]);
+                    log.error(message, e);
+
+                    if (Boolean.getBoolean("org.apache.catalina.startup.EXIT_ON_INIT_FAILURE"))
+                        throw new LifecycleException(message);
                 }
+            }
         }
     }
     
