@@ -28,7 +28,7 @@ import java.io.Serializable;
  *
  * @author Remy Maucherat
  * @author Peter Rossbach (Peter Rossbach (pero@apache.org))
- * @version $Id: ContextResourceLink.java 939350 2010-04-29 15:36:29Z kkolinko $
+ * @version $Id: ContextResourceLink.java 1061506 2011-01-20 20:54:23Z kkolinko $
  */
 
 public class ContextResourceLink extends ResourceBase implements Serializable {
@@ -49,7 +49,16 @@ public class ContextResourceLink extends ResourceBase implements Serializable {
         this.global = global;
     }
 
+    private String factory = null;
 
+    public String getFactory() {
+        return factory;
+    }
+
+    public void setFactory(String factory) {
+        this.factory = factory;
+    }    
+    
     // --------------------------------------------------------- Public Methods
 
 
