@@ -64,6 +64,11 @@ public class BackupManager extends StandardManager implements ClusterManager, Ma
      */
     private int mapSendOptions = Channel.SEND_OPTIONS_SYNCHRONIZED_ACK|Channel.SEND_OPTIONS_USE_ACK;
 
+    /**
+     * Timeout for RPC messages.
+     */
+    private long rpcTimeout = DEFAULT_REPL_TIMEOUT;
+
     /**
      * Constructor, just calls super()
      *
@@ -141,6 +146,7 @@ public class BackupManager extends StandardManager implements ClusterManager, Ma
             DeltaSession session = (DeltaSession)value;
             synchronized (session) {
                 session.access();
+                session.setPrimarySession(true);
                 session.endAccess();
             }
         }
@@ -196,7 +202,7 @@ public class BackupManager extends StandardManager implements ClusterManager, Ma
             CatalinaCluster catclust = (CatalinaCluster)cluster;
             LazyReplicatedMap map = new LazyReplicatedMap(this,
                                                           catclust.getChannel(),
-                                                          DEFAULT_REPL_TIMEOUT,
+                                                          rpcTimeout,
                                                           getMapName(),
                                                           getClassLoaders());
             map.setChannelSendOptions(mapSendOptions);
@@ -276,6 +282,14 @@ public class BackupManager extends StandardManager implements ClusterManager, Ma
         return mapSendOptions;
     }
 
+    public void setRpcTimeout(long rpcTimeout) {
+        this.rpcTimeout = rpcTimeout;
+    }
+
+    public long getRpcTimeout() {
+        return rpcTimeout;
+    }
+
     public String[] getInvalidatedSessions() {
         return new String[0];
     }
@@ -287,6 +301,8 @@ public class BackupManager extends StandardManager implements ClusterManager, Ma
         result.cluster = cluster;
         result.notifyListenersOnReplication = notifyListenersOnReplication;
         result.mapSendOptions = mapSendOptions;
+        result.maxActiveSessions = maxActiveSessions;
+        result.rpcTimeout = rpcTimeout;
         return result;
     }
 
