@@ -56,7 +56,7 @@ import java.util.concurrent.atomic.AtomicInteger;
  * track of deltas during a request.
  *
  * @author Filip Hanik
- * @version $Id: DeltaSession.java 1153318 2011-08-03 01:34:02Z kfujino $
+ * @version $Id: DeltaSession.java 1195604 2011-10-31 18:42:09Z kkolinko $
  */
 
 public class DeltaSession extends StandardSession implements Externalizable,ClusterSession,ReplicatedMapEntry {
@@ -252,7 +252,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
      *            The new session identifier
      */
     public void setId(String id) {
-        super.setId(id);
+        super.setId(id, true);
         resetDeltaRequest();
     }
 
@@ -549,6 +549,37 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
 
 
 
+    /**
+     * Check whether the Object can be distributed.
+     * The object is always distributable, if the cluster manager
+     * decides to never distribute it.
+     * @param name The name of the attribute to check
+     * @param value The value of the attribute to check
+     * @return true if the attribute is distributable, false otherwise
+     */
+    @Override
+    protected boolean isAttributeDistributable(String name, Object value) {
+        if (manager instanceof ClusterManagerBase &&
+            !((ClusterManagerBase)manager).willAttributeDistribute(name))
+            return true;
+        return super.isAttributeDistributable(name, value);
+    }
+
+    /**
+     * Exclude attributes from replication.
+     * @param name the attribute's name
+     * @return true if attribute should not be replicated
+     */
+    @Override
+    protected boolean exclude(String name) {
+
+        if (super.exclude(name))
+            return true;
+        if (manager instanceof ClusterManagerBase)
+            return !((ClusterManagerBase)manager).willAttributeDistribute(name);
+        return false;
+    }
+
     /**
      * Remove the object bound with the specified name from this session. If the
      * session does not have an object bound with this name, this method does
@@ -624,6 +655,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
 
     // -------------------------------------------- HttpSession Private Methods
 
+
     /**
      * Read a serialized version of this session object from the specified
      * object input stream.
