@@ -78,7 +78,7 @@ import org.apache.tomcat.util.modeler.Registry;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: WebappLoader.java 958543 2010-06-28 11:03:12Z rjung $
+ * @version $Id: WebappLoader.java 1066487 2011-02-02 14:48:39Z markt $
  */
 
 public class WebappLoader
@@ -671,6 +671,8 @@ public class WebappLoader
                         ((StandardContext) container).getClearReferencesStopTimerThreads());
                 classLoader.setClearReferencesThreadLocals(
                         ((StandardContext) container).getClearReferencesThreadLocals());
+                classLoader.setClearReferencesHttpClientKeepAliveThread(
+                        ((StandardContext) container).getClearReferencesHttpClientKeepAliveThread());
             }
 
             for (int i = 0; i < repositories.length; i++) {
