@@ -55,7 +55,7 @@ import org.apache.catalina.util.ServerInfo;
 * @author Malcolm Edgar
 * @author Glenn L. Nielsen
 * @author Peter Rossbach
-* @version $Id: HTMLHostManagerServlet.java 939518 2010-04-30 00:08:58Z kkolinko $
+* @version $Id: HTMLHostManagerServlet.java 1030547 2010-11-03 17:17:59Z markt $
 * @see ManagerServlet
 */
 
@@ -105,6 +105,13 @@ public final class HTMLHostManagerServlet extends HostManagerServlet {
     }
 
     
+    @Override
+    public void doPost(HttpServletRequest request, HttpServletResponse response)
+            throws IOException, ServletException {
+        doGet(request, response);
+    }
+    
+    
     /**
      * Add a host using the specified parameters.
      *
@@ -435,7 +442,7 @@ public final class HTMLHostManagerServlet extends HostManagerServlet {
         "</tr>\n" +
         "<tr>\n" +
         " <td colspan=\"2\">\n" +
-        "<form method=\"get\" action=\"{2}\">\n" +
+        "<form method=\"post\" action=\"{2}\">\n" +
         "<table cellspacing=\"0\" cellpadding=\"3\">\n" +
         "<tr>\n" +
         " <td class=\"row-right\">\n" +
