@@ -169,7 +169,7 @@ import org.apache.tomcat.util.buf.CharChunk;
  *
  * @author John Holman
  * @author Craig R. McClanahan
- * @version $Id: JNDIRealm.java 947616 2010-05-24 12:05:44Z markt $
+ * @version $Id: JNDIRealm.java 1090775 2011-04-10 11:02:22Z markt $
  */
 
 public class JNDIRealm extends RealmBase {
@@ -1211,6 +1211,11 @@ public class JNDIRealm extends RealmBase {
                                     String dn)
         throws NamingException {
 
+        // If no attributes are requested, no need to look for them
+        if (attrIds == null || attrIds.length == 0) {
+            return new User(username, dn, null, null);
+        }
+
         // Get required attributes from user entry
         Attributes attrs = null;
         try {
