@@ -32,6 +32,7 @@ import java.util.ArrayList;
 import java.util.Date;
 import java.util.Enumeration;
 import java.util.Hashtable;
+import java.util.List;
 import java.util.Locale;
 import java.util.StringTokenizer;
 import java.util.Vector;
@@ -235,7 +236,7 @@ import org.apache.catalina.util.IOTools;
  *
  * @author Martin T Dengler [root@martindengler.com]
  * @author Amy Roh
- * @version $Id: CGIServlet.java 939518 2010-04-30 00:08:58Z kkolinko $
+ * @version $Id: CGIServlet.java 1061536 2011-01-20 21:44:08Z markt $
  * @since Tomcat 4.0
  *
  */
@@ -258,7 +259,10 @@ public final class CGIServlet extends HttpServlet {
 
     /** the executable to use with the script */
     private String cgiExecutable = "perl";
-    
+
+    /** additional arguments for the executable */
+    private List<String> cgiExecutableArgs = null;
+
     /** the encoding to use for parameters */
     private String parameterEncoding =
         System.getProperty("file.encoding", "UTF-8");
@@ -317,6 +321,19 @@ public final class CGIServlet extends HttpServlet {
             cgiExecutable = getServletConfig().getInitParameter("executable");
         }
 
+        if (getServletConfig().getInitParameter("executable-arg-1") != null) {
+            List<String> args = new ArrayList<String>();
+            for (int i = 1;; i++) {
+                String arg = getServletConfig().getInitParameter(
+                        "executable-arg-" + i);
+                if (arg == null) {
+                    break;
+                }
+                args.add(arg);
+            }
+            cgiExecutableArgs = args;
+        }
+
         if (getServletConfig().getInitParameter("parameterEncoding") != null) {
             parameterEncoding = getServletConfig().getInitParameter("parameterEncoding");
         }
@@ -650,7 +667,7 @@ public final class CGIServlet extends HttpServlet {
      * <p>
      * </p>
      *
-     * @version  $Id: CGIServlet.java 939518 2010-04-30 00:08:58Z kkolinko $
+     * @version  $Id: CGIServlet.java 1061536 2011-01-20 21:44:08Z markt $
      * @since    Tomcat 4.0
      *
      */
@@ -1392,7 +1409,7 @@ public final class CGIServlet extends HttpServlet {
      * and <code>setResponse</code> methods, respectively.
      * </p>
      *
-     * @version $Id: CGIServlet.java 939518 2010-04-30 00:08:58Z kkolinko $
+     * @version $Id: CGIServlet.java 1061536 2011-01-20 21:44:08Z markt $
      */
 
     protected class CGIRunner {
@@ -1615,38 +1632,21 @@ public final class CGIServlet extends HttpServlet {
             Process proc = null;
             int bufRead = -1;
 
-            //create query arguments
-            StringBuffer cmdAndArgs = new StringBuffer();
-            if (command.indexOf(" ") < 0) {
-                cmdAndArgs.append(command);
-            } else {
-                // Spaces used as delimiter, so need to use quotes
-                cmdAndArgs.append("\"");
-                cmdAndArgs.append(command);
-                cmdAndArgs.append("\"");
+            List<String> cmdAndArgs = new ArrayList<String>();
+            if (cgiExecutable.length() != 0) {
+                cmdAndArgs.add(cgiExecutable);
             }
-
-            for (int i=0; i < params.size(); i++) {
-                cmdAndArgs.append(" ");
-                String param = (String) params.get(i);
-                if (param.indexOf(" ") < 0) {
-                    cmdAndArgs.append(param);
-                } else {
-                    // Spaces used as delimiter, so need to use quotes
-                    cmdAndArgs.append("\"");
-                    cmdAndArgs.append(param);
-                    cmdAndArgs.append("\"");
-                }
+            if (cgiExecutableArgs != null) {
+                cmdAndArgs.addAll(cgiExecutableArgs);
             }
-
-            StringBuffer command = new StringBuffer(cgiExecutable);
-            command.append(" ");
-            command.append(cmdAndArgs.toString());
-            cmdAndArgs = command;
+            cmdAndArgs.add(command);
+            cmdAndArgs.addAll(params);
 
             try {
                 rt = Runtime.getRuntime();
-                proc = rt.exec(cmdAndArgs.toString(), hashToStringArray(env), wd);
+                proc = rt.exec(
+                        cmdAndArgs.toArray(new String[cmdAndArgs.size()]),
+                        hashToStringArray(env), wd);
     
                 String sContentLength = (String) env.get("CONTENT_LENGTH");
 
