@@ -59,7 +59,7 @@ import org.apache.catalina.security.SecurityUtil;
  *
  * @author Craig R. McClanahan
  * @author Jean-Francois Arcand
- * @version $Id: StandardManager.java 942683 2010-05-10 08:39:48Z kfujino $
+ * @version $Id: StandardManager.java 1056768 2011-01-08 19:15:44Z markt $
  */
 
 public class StandardManager
@@ -481,7 +481,12 @@ public class StandardManager
     protected void doUnload() throws IOException {
 
         if (log.isDebugEnabled())
-            log.debug("Unloading persisted sessions");
+            log.debug(sm.getString("standardManager.unloading.debug"));
+
+        if (sessions.isEmpty()) {
+            log.debug(sm.getString("standardManager.unloading.nosessions"));
+            return; // nothing to do
+        }
 
         // Open an output stream to the specified pathname, if any
         File file = file();
