@@ -43,7 +43,7 @@ import org.apache.catalina.Globals;
  * from within web.xml.
  * 
  * @author David Becker
- * @version $Id: SSIFilter.java 939518 2010-04-30 00:08:58Z kkolinko $
+ * @version $Id: SSIFilter.java 964732 2010-07-16 10:23:04Z markt $
  * @see org.apache.catalina.ssi.SSIServlet
  */
 public class SSIFilter implements Filter {
@@ -59,6 +59,8 @@ public class SSIFilter implements Filter {
 	/** default pattern for ssi filter content type matching */
 	protected Pattern shtmlRegEx =
         Pattern.compile("text/x-server-parsed-html(;.*)?");
+	/** Allow exec (normally blocked for security) */
+	protected boolean allowExec = false;
 
 
     //----------------- Public methods.
@@ -87,6 +89,8 @@ public class SSIFilter implements Filter {
         if (config.getInitParameter("expires") != null)
             expires = Long.valueOf(config.getInitParameter("expires"));
 
+        allowExec = Boolean.parseBoolean(config.getInitParameter("allowExec"));
+
         if (debug > 0)
             config.getServletContext().log(
                     "SSIFilter.init() SSI invoker started with 'debug'=" + debug);
@@ -125,7 +129,7 @@ public class SSIFilter implements Filter {
                 new SSIServletExternalResolver(config.getServletContext(), req,
                         res, isVirtualWebappRelative, debug, encoding);
             SSIProcessor ssiProcessor = new SSIProcessor(ssiExternalResolver,
-                    debug);
+                    debug, allowExec);
             
             // prepare readers/writers
             Reader reader =
