@@ -98,7 +98,7 @@ import org.apache.tomcat.util.log.SystemLogHandler;
  * of how Tomcat is set up and launched as an Embedded application.
  *
  * @author Craig R. McClanahan
- * @version $Id: Embedded.java 939353 2010-04-29 15:50:43Z kkolinko $
+ * @version $Id: Embedded.java 1070204 2011-02-13 10:25:08Z markt $
  */
 
 public class Embedded  extends StandardService implements Lifecycle {
@@ -906,7 +906,7 @@ public class Embedded  extends StandardService implements Lifecycle {
                     (javax.naming.Context.INITIAL_CONTEXT_FACTORY,
                      "org.apache.naming.java.javaURLContextFactory");
             } else {
-                log.debug( "INITIAL_CONTEXT_FACTORY alread set " + value );
+                log.debug( "INITIAL_CONTEXT_FACTORY already set " + value );
             }
         }
     }
