@@ -86,7 +86,7 @@ public final class TldConfig  implements LifecycleListener {
         noTldJars.add("el-api.jar");
         noTldJars.add("jasper.jar");
         noTldJars.add("jasper-el.jar");
-        noTldJars.add("jasper-jdt.jar");
+        noTldJars.add("ecj-3.7.jar");
         noTldJars.add("jsp-api.jar");
         noTldJars.add("servlet-api.jar");
         noTldJars.add("tomcat-coyote.jar");
