@@ -24,6 +24,7 @@ import java.io.InterruptedIOException;
 import java.io.OutputStream;
 import java.net.InetAddress;
 import java.net.Socket;
+import java.security.NoSuchProviderException;
 import java.security.cert.CertificateFactory;
 import java.security.cert.X509Certificate;
 
@@ -344,6 +345,20 @@ public class AjpProcessor implements ActionHook {
     public void setKeepAliveTimeout(int timeout) { keepAliveTimeout = timeout; }
 
 
+    /**
+     * When client certificate information is presented in a form other than
+     * instances of {@link java.security.cert.X509Certificate} it needs to be
+     * converted before it can be used and this property controls which JSSE
+     * provider is used to perform the conversion. For example it is used with
+     * the AJP connectors, the HTTP APR connector and with the
+     * {@link org.apache.catalina.valves.SSLValve}. If not specified, the
+     * default provider will be used. 
+     */
+    protected String clientCertProvider = null;
+    public String getClientCertProvider() { return clientCertProvider; }
+    public void setClientCertProvider(String s) { this.clientCertProvider = s; }
+
+
     // --------------------------------------------------------- Public Methods
 
 
@@ -408,11 +423,13 @@ public class AjpProcessor implements ActionHook {
                     }
                     continue;
                 } else if(type != Constants.JK_AJP13_FORWARD_REQUEST) {
-                    // Usually the servlet didn't read the previous request body
-                    if(log.isDebugEnabled()) {
-                        log.debug("Unexpected message: "+type);
+                    // Unexpected packet type. Unread body packets should have
+                    // been swallowed in finish().
+                    if (log.isDebugEnabled()) {
+                        log.debug("Unexpected message: " + type);
                     }
-                    continue;
+                    error = true;
+                    break;
                 }
 
                 request.setStartTime(System.currentTimeMillis());
@@ -428,15 +445,17 @@ public class AjpProcessor implements ActionHook {
             }
 
             // Setting up filters, and parse some request headers
-            rp.setStage(org.apache.coyote.Constants.STAGE_PREPARE);
-            try {
-                prepareRequest();
-            } catch (Throwable t) {
-                log.debug(sm.getString("ajpprocessor.request.prepare"), t);
-                // 400 - Internal Server Error
-                response.setStatus(400);
-                adapter.log(request, response, 0);
-                error = true;
+            if (!error) {
+                rp.setStage(org.apache.coyote.Constants.STAGE_PREPARE);
+                try {
+                    prepareRequest();
+                } catch (Throwable t) {
+                    log.debug(sm.getString("ajpprocessor.request.prepare"), t);
+                    // 400 - Internal Server Error
+                    response.setStatus(400);
+                    adapter.log(request, response, 0);
+                    error = true;
+                }
             }
 
             // Process the request in the adapter
@@ -560,8 +579,13 @@ public class AjpProcessor implements ActionHook {
                             certData.getLength());
                 // Fill the elements.
                 try {
-                    CertificateFactory cf =
-                        CertificateFactory.getInstance("X.509");
+                    CertificateFactory cf;
+                    if (clientCertProvider == null) {
+                        cf = CertificateFactory.getInstance("X.509");
+                    } else {
+                        cf = CertificateFactory.getInstance("X.509",
+                                clientCertProvider);
+                    }
                     while(bais.available() > 0) {
                         X509Certificate cert = (X509Certificate)
                             cf.generateCertificate(bais);
@@ -578,6 +602,9 @@ public class AjpProcessor implements ActionHook {
                 } catch (java.security.cert.CertificateException e) {
                     log.error(sm.getString("ajpprocessor.certs.fail"), e);
                     return;
+                } catch (NoSuchProviderException e) {
+                    log.error(sm.getString("ajpprocessor.certs.fail"), e);
+                    return;
                 }
                 request.setAttribute(JIoEndpoint.CERTIFICATE_KEY, jsseCerts);
             }
@@ -819,7 +846,6 @@ public class AjpProcessor implements ActionHook {
                     secret = true;
                     if (!tmpMB.equals(requiredSecret)) {
                         response.setStatus(403);
-                        adapter.log(request, response, 0);
                         error = true;
                     }
                 }
@@ -836,7 +862,6 @@ public class AjpProcessor implements ActionHook {
         // Check if secret was submitted if required
         if ((requiredSecret != null) && !secret) {
             response.setStatus(403);
-            adapter.log(request, response, 0);
             error = true;
         }
 
@@ -870,6 +895,9 @@ public class AjpProcessor implements ActionHook {
         MessageBytes valueMB = request.getMimeHeaders().getValue("host");
         parseHost(valueMB);
 
+        if (error) {
+            adapter.log(request, response, 0);
+        }
     }
 
 
@@ -885,7 +913,6 @@ public class AjpProcessor implements ActionHook {
                 request.serverName().duplicate(request.localName());
             } catch (IOException e) {
                 response.setStatus(400);
-                adapter.log(request, response, 0);
                 error = true;
             }
             return;
@@ -937,7 +964,6 @@ public class AjpProcessor implements ActionHook {
                     error = true;
                     // 400 - Bad request
                     response.setStatus(400);
-                    adapter.log(request, response, 0);
                     break;
                 }
                 port = port + (charValue * mult);
@@ -1038,6 +1064,11 @@ public class AjpProcessor implements ActionHook {
 
         finished = true;
 
+        // Swallow the unread body packet if present
+        if (first && request.getContentLengthLong() > 0) {
+            receive();
+        }
+        
         // Add the end message
         output.write(endMessageArray);
 
@@ -1088,7 +1119,7 @@ public class AjpProcessor implements ActionHook {
             return false;
         }
 
-        bodyMessage.getBytes(bodyBytes);
+        bodyMessage.getBodyBytes(bodyBytes);
         empty = false;
         return true;
     }
@@ -1166,8 +1197,10 @@ public class AjpProcessor implements ActionHook {
      */
     protected void flush()
         throws IOException {
-        // Send the flush message
-        output.write(flushMessageArray);
+        if (!finished) {
+            // Send the flush message
+            output.write(flushMessageArray);
+        }
     }
 
 
