@@ -972,8 +972,9 @@ public class Http11AprProcessor implements ActionHook {
         } catch (Throwable t) {
             log.error(sm.getString("http11processor.request.finish"), t);
             // 500 - Internal Server Error
+            // Can't add a 500 to the access log since that has already been
+            // written in the Adapter.service method.
             response.setStatus(500);
-            adapter.log(request, response, 0);
             error = true;
         }
         try {
@@ -1327,7 +1328,6 @@ public class Http11AprProcessor implements ActionHook {
             error = true;
             // Send 505; Unsupported HTTP version
             response.setStatus(505);
-            adapter.log(request, response, 0);
         }
 
         MessageBytes methodMB = request.method();
@@ -1425,7 +1425,6 @@ public class Http11AprProcessor implements ActionHook {
                     error = true;
                     // 501 - Unimplemented
                     response.setStatus(501);
-                    adapter.log(request, response, 0);
                 }
                 startPos = commaPos + 1;
                 commaPos = transferEncodingValue.indexOf(',', startPos);
@@ -1437,7 +1436,6 @@ public class Http11AprProcessor implements ActionHook {
                 error = true;
                 // 501 - Unimplemented
                 response.setStatus(501);
-                adapter.log(request, response, 0);
             }
         }
 
@@ -1456,7 +1454,6 @@ public class Http11AprProcessor implements ActionHook {
             error = true;
             // 400 - Bad request
             response.setStatus(400);
-            adapter.log(request, response, 0);
         }
 
         parseHost(valueMB);
@@ -1476,7 +1473,10 @@ public class Http11AprProcessor implements ActionHook {
         }
         // Advertise comet support through a request attribute
         request.setAttribute("org.apache.tomcat.comet.support", Boolean.TRUE);
-        
+
+        if (error) {
+            adapter.log(request, response, 0);
+        }
     }
 
 
@@ -1539,7 +1539,6 @@ public class Http11AprProcessor implements ActionHook {
                     error = true;
                     // 400 - Bad request
                     response.setStatus(400);
-                    adapter.log(request, response, 0);
                     break;
                 }
                 port = port + (charValue * mult);
