@@ -986,8 +986,9 @@ public class Http11NioProcessor implements ActionHook {
         } catch (Throwable t) {
             log.error(sm.getString("http11processor.request.finish"), t);
             // 500 - Internal Server Error
+            // Can't add a 500 to the access log since that has already been
+            // written in the Adapter.service method.
             response.setStatus(500);
-            adapter.log(request, response, 0);
             error = true;
         }
         try {
@@ -1322,7 +1323,6 @@ public class Http11NioProcessor implements ActionHook {
             error = true;
             // Send 505; Unsupported HTTP version
             response.setStatus(505);
-            adapter.log(request, response, 0);
         }
 
         MessageBytes methodMB = request.method();
@@ -1420,7 +1420,6 @@ public class Http11NioProcessor implements ActionHook {
                     error = true;
                     // 501 - Unimplemented
                     response.setStatus(501);
-                    adapter.log(request, response, 0);
                 }
                 startPos = commaPos + 1;
                 commaPos = transferEncodingValue.indexOf(',', startPos);
@@ -1432,7 +1431,6 @@ public class Http11NioProcessor implements ActionHook {
                 error = true;
                 // 501 - Unimplemented
                 response.setStatus(501);
-                adapter.log(request, response, 0);
             }
         }
 
@@ -1451,7 +1449,6 @@ public class Http11NioProcessor implements ActionHook {
             error = true;
             // 400 - Bad request
             response.setStatus(400);
-            adapter.log(request, response, 0);
         }
 
         parseHost(valueMB);
@@ -1473,6 +1470,9 @@ public class Http11NioProcessor implements ActionHook {
         // Advertise comet timeout support
         request.setAttribute("org.apache.tomcat.comet.timeout.support", Boolean.TRUE);
 
+        if (error) {
+            adapter.log(request, response, 0);
+        }
     }
 
 
@@ -1535,7 +1535,6 @@ public class Http11NioProcessor implements ActionHook {
                     error = true;
                     // 400 - Bad request
                     response.setStatus(400);
-                    adapter.log(request, response, 0);
                     break;
                 }
                 port = port + (charValue * mult);
