@@ -799,7 +799,7 @@ public class Http11Processor implements ActionHook {
 
         boolean keptAlive = false;
 
-        while (started && !error && keepAlive) {
+        while (started && !error && keepAlive && !endpoint.isPaused()) {
 
             // Parsing the request header
             try {
@@ -829,6 +829,7 @@ public class Http11Processor implements ActionHook {
                 }
                 // 400 - Bad Request
                 response.setStatus(400);
+                adapter.log(request, response, 0);
                 error = true;
             }
 
@@ -843,6 +844,7 @@ public class Http11Processor implements ActionHook {
                     }
                     // 400 - Internal Server Error
                     response.setStatus(400);
+                    adapter.log(request, response, 0);
                     error = true;
                 }
             }
@@ -871,6 +873,7 @@ public class Http11Processor implements ActionHook {
                     log.error(sm.getString("http11processor.request.process"), t);
                     // 500 - Internal Server Error
                     response.setStatus(500);
+                    adapter.log(request, response, 0);
                     error = true;
                 }
             }
@@ -890,6 +893,7 @@ public class Http11Processor implements ActionHook {
                 log.error(sm.getString("http11processor.request.finish"), t);
                 // 500 - Internal Server Error
                 response.setStatus(500);
+                // No access logging since after service method 
                 error = true;
             }
             try {
@@ -1344,6 +1348,9 @@ public class Http11Processor implements ActionHook {
             contentDelimitation = true;
         }
 
+        if (error) {
+            adapter.log(request, response, 0);
+        }
     }
 
 
@@ -1423,16 +1430,42 @@ public class Http11Processor implements ActionHook {
 
 
     /**
-     * Check for compression
+     * Check if the resource could be compressed, if the client supports it.
      */
     private boolean isCompressable() {
 
-        // Nope Compression could works in HTTP 1.0 also
-        // cf: mod_deflate
+        // Check if content is not already gzipped
+        MessageBytes contentEncodingMB =
+            response.getMimeHeaders().getValue("Content-Encoding");
 
-        // Compression only since HTTP 1.1
-        // if (! http11)
-        //    return false;
+        if ((contentEncodingMB != null)
+            && (contentEncodingMB.indexOf("gzip") != -1))
+            return false;
+
+        // If force mode, always compress (test purposes only)
+        if (compressionLevel == 2)
+           return true;
+
+        // Check if sufficient length to trigger the compression
+        long contentLength = response.getContentLengthLong();
+        if ((contentLength == -1)
+            || (contentLength > compressionMinSize)) {
+            // Check for compatible MIME-TYPE
+            if (compressableMimeTypes != null) {
+                return (startsWithStringArray(compressableMimeTypes,
+                                              response.getContentType()));
+            }
+        }
+
+        return false;
+    }
+
+
+    /**
+     * Check if compression should be used for this resource. Already checked
+     * that the resource could be compressed if the client supports it.
+     */
+    private boolean useCompression() {
 
         // Check if browser support gzip encoding
         MessageBytes acceptEncodingMB =
@@ -1442,15 +1475,7 @@ public class Http11Processor implements ActionHook {
             || (acceptEncodingMB.indexOf("gzip") == -1))
             return false;
 
-        // Check if content is not allready gzipped
-        MessageBytes contentEncodingMB =
-            response.getMimeHeaders().getValue("Content-Encoding");
-
-        if ((contentEncodingMB != null)
-            && (contentEncodingMB.indexOf("gzip") != -1))
-            return false;
-
-        // If force mode, allways compress (test purposes only)
+        // If force mode, always compress (test purposes only)
         if (compressionLevel == 2)
            return true;
 
@@ -1468,21 +1493,10 @@ public class Http11Processor implements ActionHook {
             }
         }
 
-        // Check if suffisant len to trig the compression
-        long contentLength = response.getContentLengthLong();
-        if ((contentLength == -1)
-            || (contentLength > compressionMinSize)) {
-            // Check for compatible MIME-TYPE
-            if (compressableMimeTypes != null) {
-                return (startsWithStringArray(compressableMimeTypes,
-                                              response.getContentType()));
-            }
-        }
-
-        return false;
+        return true;
     }
 
-
+    
     /**
      * When committing the response, we have to validate the set of headers, as
      * well as setup the response filters.
@@ -1520,10 +1534,13 @@ public class Http11Processor implements ActionHook {
         }
 
         // Check for compression
+        boolean isCompressable = false;
         boolean useCompression = false;
         if (entityBody && (compressionLevel > 0)) {
-            useCompression = isCompressable();
-
+            isCompressable = isCompressable();
+            if (isCompressable) {
+                useCompression = useCompression();
+            }
             // Change content-length to -1 to force chunking
             if (useCompression) {
                 response.setContentLength(-1);
@@ -1566,6 +1583,9 @@ public class Http11Processor implements ActionHook {
         if (useCompression) {
             outputBuffer.addActiveFilter(outputFilters[Constants.GZIP_FILTER]);
             headers.setValue("Content-Encoding").setString("gzip");
+        }
+        // If it might be compressed, set the Vary header
+        if (isCompressable) {
             // Make Proxies happy via Vary (from mod_deflate)
             MessageBytes vary = headers.getValue("Vary");
             if (vary == null) {
