@@ -32,7 +32,7 @@ import org.apache.el.util.MessageFactory;
  * A helper class that implements the EL Specification
  * 
  * @author Jacob Hookom [jacob@hookom.net]
- * @version $Change: 181177 $$DateTime: 2001/06/26 08:45:09 $$Author: markt $
+ * @version $Id: ELSupport.java 939519 2010-04-30 00:12:23Z kkolinko $
  */
 public class ELSupport {
 
@@ -143,22 +143,38 @@ public class ELSupport {
      * @param type
      * @return
      */
-    public final static Enum coerceToEnum(final Object obj, Class type) {
+    public final static Enum<?> coerceToEnum(final Object obj, Class type) {
         if (obj == null || "".equals(obj)) {
             return null;
         }
-        if (obj.getClass().isEnum()) {
+        if (type.isAssignableFrom(obj.getClass())) {
             return (Enum) obj;
         }
-        return Enum.valueOf(type, obj.toString());
+        
+        if (!(obj instanceof String)) {
+            throw new ELException(MessageFactory.get("error.convert",
+                    obj, obj.getClass(), type));
+        }
+
+        Enum<?> result;
+        try {
+             result = Enum.valueOf(type, (String) obj);
+        } catch (IllegalArgumentException iae) {
+            throw new ELException(MessageFactory.get("error.convert",
+                    obj, obj.getClass(), type));
+        }
+        return result;
     }
 
     /**
-     * @param obj
-     * @return
+     * Convert an object to Boolean.
+     * Null and empty string are false.
+     * @param obj the object to convert
+     * @return the Boolean value of the object
+     * @throws ELException if object is not Boolean or String
      */
     public final static Boolean coerceToBoolean(final Object obj)
-            throws IllegalArgumentException {
+            throws ELException {
         if (obj == null || "".equals(obj)) {
             return Boolean.FALSE;
         }
@@ -169,12 +185,12 @@ public class ELSupport {
             return Boolean.valueOf((String) obj);
         }
 
-        throw new IllegalArgumentException(MessageFactory.get("error.convert",
+        throw new ELException(MessageFactory.get("error.convert",
                 obj, obj.getClass(), Boolean.class));
     }
 
     public final static Character coerceToCharacter(final Object obj)
-            throws IllegalArgumentException {
+            throws ELException {
         if (obj == null || "".equals(obj)) {
             return new Character((char) 0);
         }
@@ -189,7 +205,7 @@ public class ELSupport {
             return (Character) obj;
         }
 
-        throw new IllegalArgumentException(MessageFactory.get("error.convert",
+        throw new ELException(MessageFactory.get("error.convert",
                 obj, objType, Character.class));
     }
 
@@ -209,7 +225,7 @@ public class ELSupport {
     }
 
     protected final static Number coerceToNumber(final Number number,
-            final Class type) throws IllegalArgumentException {
+            final Class<?> type) throws ELException {
         if (Long.TYPE == type || Long.class.equals(type)) {
             return new Long(number.longValue());
         }
@@ -250,12 +266,12 @@ public class ELSupport {
             return number;
         }
 
-        throw new IllegalArgumentException(MessageFactory.get("error.convert",
+        throw new ELException(MessageFactory.get("error.convert",
                 number, number.getClass(), type));
     }
 
-    public final static Number coerceToNumber(final Object obj, final Class type)
-            throws IllegalArgumentException {
+    public final static Number coerceToNumber(final Object obj,
+            final Class<?> type) throws ELException {
         if (obj == null || "".equals(obj)) {
             return coerceToNumber(ZERO, type);
         }
@@ -271,38 +287,78 @@ public class ELSupport {
                     .charValue()), type);
         }
 
-        throw new IllegalArgumentException(MessageFactory.get("error.convert",
+        throw new ELException(MessageFactory.get("error.convert",
                 obj, obj.getClass(), type));
     }
 
     protected final static Number coerceToNumber(final String val,
-            final Class type) throws IllegalArgumentException {
+            final Class<?> type) throws ELException {
         if (Long.TYPE == type || Long.class.equals(type)) {
-            return Long.valueOf(val);
+            try {
+                return Long.valueOf(val);
+            } catch (NumberFormatException nfe) {
+                throw new ELException(MessageFactory.get("error.convert",
+                        val, String.class, type));
+            }
         }
         if (Integer.TYPE == type || Integer.class.equals(type)) {
-            return Integer.valueOf(val);
+            try {
+                return Integer.valueOf(val);
+            } catch (NumberFormatException nfe) {
+                throw new ELException(MessageFactory.get("error.convert",
+                        val, String.class, type));
+            }
         }
         if (Double.TYPE == type || Double.class.equals(type)) {
-            return Double.valueOf(val);
+            try {
+                return Double.valueOf(val);
+            } catch (NumberFormatException nfe) {
+                throw new ELException(MessageFactory.get("error.convert",
+                        val, String.class, type));
+            }
         }
         if (BigInteger.class.equals(type)) {
-            return new BigInteger(val);
+            try {
+                return new BigInteger(val);
+            } catch (NumberFormatException nfe) {
+                throw new ELException(MessageFactory.get("error.convert",
+                        val, String.class, type));
+            }
         }
         if (BigDecimal.class.equals(type)) {
-            return new BigDecimal(val);
+            try {
+                return new BigDecimal(val);
+            } catch (NumberFormatException nfe) {
+                throw new ELException(MessageFactory.get("error.convert",
+                        val, String.class, type));
+            }
         }
         if (Byte.TYPE == type || Byte.class.equals(type)) {
-            return Byte.valueOf(val);
+            try {
+                return Byte.valueOf(val);
+            } catch (NumberFormatException nfe) {
+                throw new ELException(MessageFactory.get("error.convert",
+                        val, String.class, type));
+            }
         }
         if (Short.TYPE == type || Short.class.equals(type)) {
-            return Short.valueOf(val);
+            try {
+                return Short.valueOf(val);
+            } catch (NumberFormatException nfe) {
+                throw new ELException(MessageFactory.get("error.convert",
+                        val, String.class, type));
+            }
         }
         if (Float.TYPE == type || Float.class.equals(type)) {
-            return Float.valueOf(val);
+            try {
+                return Float.valueOf(val);
+            } catch (NumberFormatException nfe) {
+                throw new ELException(MessageFactory.get("error.convert",
+                        val, String.class, type));
+            }
         }
 
-        throw new IllegalArgumentException(MessageFactory.get("error.convert",
+        throw new ELException(MessageFactory.get("error.convert",
                 val, String.class, type));
     }
 
@@ -323,7 +379,7 @@ public class ELSupport {
     }
 
     public final static void checkType(final Object obj, final Class type)
-        throws IllegalArgumentException {
+        throws ELException {
         if (String.class.equals(type)) {
             coerceToString(obj);
         }
@@ -341,8 +397,8 @@ public class ELSupport {
         }
     }
 
-    public final static Object coerceToType(final Object obj, final Class type)
-            throws IllegalArgumentException {
+    public final static Object coerceToType(final Object obj,
+            final Class<?> type) throws ELException {
         if (type == null || Object.class.equals(type) ||
                 (obj != null && type.isAssignableFrom(obj.getClass()))) {
             return obj;
@@ -375,7 +431,7 @@ public class ELSupport {
                 return editor.getValue();
             }
         }
-        throw new IllegalArgumentException(MessageFactory.get("error.convert",
+        throw new ELException(MessageFactory.get("error.convert",
                 obj, obj.getClass(), type));
     }
 
