@@ -42,13 +42,12 @@ import org.apache.el.parser.AstValue;
 import org.apache.el.parser.ELParser;
 import org.apache.el.parser.Node;
 import org.apache.el.parser.NodeVisitor;
-import org.apache.el.parser.ParseException;
 import org.apache.el.util.ConcurrentCache;
 import org.apache.el.util.MessageFactory;
 
 /**
  * @author Jacob Hookom [jacob@hookom.net]
- * @version $Id: ExpressionBuilder.java 1027187 2010-10-25 16:37:14Z markt $
+ * @version $Id: ExpressionBuilder.java 1142213 2011-07-02 12:11:27Z markt $
  */
 public final class ExpressionBuilder implements NodeVisitor {
 
@@ -143,8 +142,9 @@ public final class ExpressionBuilder implements NodeVisitor {
                     n = n.jjtGetChild(0);
                 }
                 cache.put(expr, n);
-            } catch (ParseException pe) {
-                throw new ELException("Error Parsing: " + expr, pe);
+            } catch (Exception e) {
+                throw new ELException(
+                        MessageFactory.get("error.parseFail", expr), e);
             }
         }
         return n;
