@@ -19,14 +19,16 @@ package org.apache.jasper;
 
 import java.io.BufferedReader;
 import java.io.CharArrayWriter;
+import java.io.EOFException;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileNotFoundException;
 import java.io.FileOutputStream;
-import java.io.FileReader;
-import java.io.FileWriter;
 import java.io.IOException;
+import java.io.InputStreamReader;
+import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
+import java.io.Reader;
 import java.io.Writer;
 import java.net.MalformedURLException;
 import java.net.URL;
@@ -64,20 +66,17 @@ import org.apache.tools.ant.util.FileUtils;
  *
  * It can be used as an Ant task using:
  * <pre>
- *   &lt;taskdef classname="org.apache.jasper.JspC" name="jasper2" &gt;
+ *   &lt;taskdef classname="org.apache.jasper.JspC" name="jasper" &gt;
  *      &lt;classpath&gt;
  *          &lt;pathelement location="${java.home}/../lib/tools.jar"/&gt;
- *          &lt;fileset dir="${ENV.CATALINA_HOME}/server/lib"&gt;
- *              &lt;include name="*.jar"/&gt;
- *          &lt;/fileset&gt;
- *          &lt;fileset dir="${ENV.CATALINA_HOME}/common/lib"&gt;
+ *          &lt;fileset dir="${ENV.CATALINA_HOME}/lib"&gt;
  *              &lt;include name="*.jar"/&gt;
  *          &lt;/fileset&gt;
  *          &lt;path refid="myjars"/&gt;
  *       &lt;/classpath&gt;
  *  &lt;/taskdef&gt;
  *
- *  &lt;jasper2 verbose="0"
+ *  &lt;jasper verbose="0"
  *           package="my.package"
  *           uriroot="${webapps.dir}/${webapp.name}"
  *           webXmlFragment="${build.dir}/generated_web.xml"
@@ -112,6 +111,8 @@ public class JspC implements Options {
     protected static final String SWITCH_FILE_WEBAPP = "-webapp";
     protected static final String SWITCH_WEBAPP_INC = "-webinc";
     protected static final String SWITCH_WEBAPP_XML = "-webxml";
+    protected static final String SWITCH_WEBAPP_XML_ENCODING = "-webxmlencoding";
+    protected static final String SWITCH_ADD_WEBAPP_XML_MAPPINGS = "-addwebxmlmappings";
     protected static final String SWITCH_MAPPED = "-mapped";
     protected static final String SWITCH_XPOWERED_BY = "-xpoweredBy";
     protected static final String SWITCH_TRIM_SPACES = "-trimSpaces";
@@ -198,6 +199,7 @@ public class JspC implements Options {
     // Generation of web.xml fragments
     protected String webxmlFile;
     protected int webxmlLevel;
+    protected String webxmlEncoding;
     protected boolean addWebXmlMappings = false;
 
     protected Writer mapout;
@@ -251,6 +253,12 @@ public class JspC implements Options {
         }
     }
 
+    /**
+     * Apply command-line arguments.
+     * 
+     * @param arg
+     *            The arguments
+     */
     public void setArgs(String[] arg) throws JasperException {
         args = arg;
         String tok;
@@ -292,6 +300,10 @@ public class JspC implements Options {
                 if (webxmlFile != null) {
                     webxmlLevel = ALL_WEBXML;
                 }
+            } else if (tok.equals(SWITCH_WEBAPP_XML_ENCODING)) {
+                setWebXmlEncoding(nextArg());
+            } else if (tok.equals(SWITCH_ADD_WEBAPP_XML_MAPPINGS)) {
+                setAddWebXmlMappings(true);
             } else if (tok.equals(SWITCH_MAPPED)) {
                 mappedFile = true;
             } else if (tok.equals(SWITCH_XPOWERED_BY)) {
@@ -357,60 +369,101 @@ public class JspC implements Options {
         }
     }
 
+    /**
+     * In JspC this always returns <code>true</code>.
+     * {@inheritDoc}
+     */
     public boolean getKeepGenerated() {
         // isn't this why we are running jspc?
         return true;
     }
 
+    /**
+     * {@inheritDoc}
+     */
     public boolean getTrimSpaces() {
         return trimSpaces;
     }
 
+    /**
+     * Sets the option to trim white spaces between directives or actions.
+     */
     public void setTrimSpaces(boolean ts) {
         this.trimSpaces = ts;
     }
 
+    /**
+     * {@inheritDoc}
+     */
     public boolean isPoolingEnabled() {
         return poolingEnabled;
     }
 
+    /**
+     * Sets the option to enable the tag handler pooling.
+     */
     public void setPoolingEnabled(boolean poolingEnabled) {
         this.poolingEnabled = poolingEnabled;
     }
 
+    /**
+     * {@inheritDoc}
+     */
     public boolean isXpoweredBy() {
         return xpoweredBy;
     }
 
+    /**
+     * Sets the option to enable generation of X-Powered-By response header.
+     */
     public void setXpoweredBy(boolean xpoweredBy) {
         this.xpoweredBy = xpoweredBy;
     }
 
+    /**
+     * In JspC this always returns <code>true</code>.
+     * {@inheritDoc}
+     */
     public boolean getDisplaySourceFragment() {
         return true;
     }
-    
+
+    /**
+     * {@inheritDoc}
+     */
     public boolean getErrorOnUseBeanInvalidClassAttribute() {
         return errorOnUseBeanInvalidClassAttribute;
     }
 
+    /**
+     * Sets the option to issue a compilation error if the class attribute
+     * specified in useBean action is invalid.
+     */
     public void setErrorOnUseBeanInvalidClassAttribute(boolean b) {
         errorOnUseBeanInvalidClassAttribute = b;
     }
 
+    /**
+     * @deprecated Removed in Tomcat 7
+     */
+    @Deprecated
     public int getTagPoolSize() {
         return Constants.MAX_POOL_SIZE;
     }
 
     /**
-     * Are we supporting HTML mapped servlets?
+     * {@inheritDoc}
      */
     public boolean getMappedFile() {
         return mappedFile;
     }
 
-    // Off-line compiler, no need for security manager
+    /**
+     * @deprecated Removed in Tomcat 7
+     */
+    @Deprecated
     public Object getProtectionDomain() {
+        // Off-line compiler, no need for security manager
         return null;
     }
 
@@ -422,23 +475,31 @@ public class JspC implements Options {
         return true;
     }
 
+    /**
+     * Sets the option to include debug information in compiled class.
+     */
     public void setClassDebugInfo( boolean b ) {
         classDebugInfo=b;
     }
 
+    /**
+     * {@inheritDoc}
+     */
     public boolean getClassDebugInfo() {
         // compile with debug info
         return classDebugInfo;
     }
 
-     /**
-      * @see Options#isCaching()
+    /**
+     * {@inheritDoc}
      */
     public boolean isCaching() {
         return caching;
     }
 
     /**
+     * Sets the option to enable caching.
+     * 
      * @see Options#isCaching()
      */
     public void setCaching(boolean caching) {
@@ -446,57 +507,71 @@ public class JspC implements Options {
     }
 
     /**
-     * @see Options#getCache()
+     * {@inheritDoc}
      */
     public Map getCache() {
         return cache;
     }
 
     /**
-     * Background compilation check intervals in seconds
+     * In JspC this always returns <code>0</code>.
+     * {@inheritDoc}
      */
     public int getCheckInterval() {
         return 0;
     }
 
     /**
-     * Modification test interval.
+     * In JspC this always returns <code>0</code>.
+     * {@inheritDoc}
      */
     public int getModificationTestInterval() {
         return 0;
     }
 
+
     /**
-     * Is Jasper being used in development mode?
+     * In JspC this always returns <code>false</code>.
+     * {@inheritDoc}
+     */
+    public boolean getRecompileOnFail() {
+        return false;
+    }
+    
+    
+    /**
+     * In JspC this always returns <code>false</code>.
+     * {@inheritDoc}
      */
     public boolean getDevelopment() {
         return false;
     }
 
     /**
-     * Is the generation of SMAP info for JSR45 debuggin suppressed?
+     * {@inheritDoc}
      */
     public boolean isSmapSuppressed() {
         return smapSuppressed;
     }
 
     /**
-     * Set smapSuppressed flag.
+     * Sets smapSuppressed flag.
      */
     public void setSmapSuppressed(boolean smapSuppressed) {
         this.smapSuppressed = smapSuppressed;
     }
 
-    
     /**
-     * Should SMAP info for JSR45 debugging be dumped to a file?
+     * {@inheritDoc}
      */
     public boolean isSmapDumped() {
         return smapDumped;
     }
 
     /**
-     * Set smapSuppressed flag.
+     * Sets smapDumped flag.
+     * 
+     * @see Options#isSmapDumped()
      */
     public void setSmapDumped(boolean smapDumped) {
         this.smapDumped = smapDumped;
@@ -515,10 +590,7 @@ public class JspC implements Options {
     }
 
     /**
-     * Indicates whether text strings are to be generated as char arrays.
-     *
-     * @return true if text strings are to be generated as char arrays, false
-     * otherwise
+     * {@inheritDoc}
      */
     public boolean genStringAsCharArray() {
         return genStringAsCharArray;
@@ -526,81 +598,105 @@ public class JspC implements Options {
 
     /**
      * Sets the class-id value to be sent to Internet Explorer when using
-     * <jsp:plugin> tags.
-     *
-     * @param ieClassId Class-id value
+     * &lt;jsp:plugin&gt; tags.
+     * 
+     * @param ieClassId
+     *            Class-id value
      */
     public void setIeClassId(String ieClassId) {
         this.ieClassId = ieClassId;
     }
 
     /**
-     * Gets the class-id value that is sent to Internet Explorer when using
-     * <jsp:plugin> tags.
-     *
-     * @return Class-id value
+     * {@inheritDoc}
      */
     public String getIeClassId() {
         return ieClassId;
     }
 
+    /**
+     * {@inheritDoc}
+     */
     public File getScratchDir() {
         return scratchDir;
     }
 
+    /**
+     * @deprecated Removed in Tomcat 7
+     */
+    @Deprecated
     public Class getJspCompilerPlugin() {
        // we don't compile, so this is meanlingless
         return null;
     }
 
+    /**
+     * @deprecated Removed in Tomcat 7
+     */
+    @Deprecated
     public String getJspCompilerPath() {
        // we don't compile, so this is meanlingless
         return null;
     }
 
     /**
-     * Compiler to use.
+     * {@inheritDoc}
      */
     public String getCompiler() {
         return compiler;
     }
 
+    /**
+     * Sets the option to determine what compiler to use.
+     * 
+     * @see Options#getCompiler()
+     */
     public void setCompiler(String c) {
         compiler=c;
     }
 
     /**
-     * Compiler class name to use.
+     * {@inheritDoc}
      */
     public String getCompilerClassName() {
         return null;
     }
-    
+
     /**
-     * @see Options#getCompilerTargetVM
+     * {@inheritDoc}
      */
     public String getCompilerTargetVM() {
         return compilerTargetVM;
     }
 
+    /**
+     * Sets the compiler target VM.
+     * 
+     * @see Options#getCompilerTargetVM()
+     */
     public void setCompilerTargetVM(String vm) {
         compilerTargetVM = vm;
     }
 
     /**
-     * @see Options#getCompilerSourceVM()
+     * {@inheritDoc}
      */
      public String getCompilerSourceVM() {
          return compilerSourceVM;
      }
-        
-    /**
-     * @see Options#getCompilerSourceVM()
-     */
+
+     /**
+      * Sets the compiler source VM.
+      * 
+      * @see Options#getCompilerSourceVM()
+      */
     public void setCompilerSourceVM(String vm) {
         compilerSourceVM = vm;
     }
 
+    /**
+     * {@inheritDoc}
+     */
     public TldLocationsCache getTldLocationsCache() {
         return tldLocationsCache;
     }
@@ -625,16 +721,26 @@ public class JspC implements Options {
         javaEncoding = encodingName;
     }
 
+    /**
+     * {@inheritDoc}
+     */
     public boolean getFork() {
         return false;
     }
 
+    /**
+     * {@inheritDoc}
+     */
     public String getClassPath() {
         if( classPath != null )
             return classPath;
         return System.getProperty("java.class.path");
     }
 
+    /**
+     * Sets the classpath used while compiling the servlets generated from JSP
+     * files
+     */
     public void setClassPath(String s) {
         classPath=s;
     }
@@ -666,7 +772,7 @@ public class JspC implements Options {
     }
 
     /**
-     * Sets the project.
+     * Sets the Ant project.
      *
      * @param theProject The project
      */
@@ -675,7 +781,7 @@ public class JspC implements Options {
     }
 
     /**
-     * Returns the project: may be null if not running
+     * Returns the project: may be <code>null</code> if not running
      * inside an Ant project.
      *
      * @return The project
@@ -686,7 +792,7 @@ public class JspC implements Options {
 
     /**
      * Base dir for the webapp. Used to generate class names and resolve
-     * includes
+     * includes.
      */
     public void setUriroot( String s ) {
         if( s==null ) {
@@ -760,6 +866,9 @@ public class JspC implements Options {
         }
     }
 
+    /**
+     * Sets the package name to be used for the generated servlet classes.
+     */
     public void setPackage( String p ) {
         targetPackage=p;
     }
@@ -789,32 +898,73 @@ public class JspC implements Options {
         webxmlLevel=ALL_WEBXML;
     }
 
+    /**
+     * Sets the encoding to be used to read and write web.xml files.
+     * 
+     * <p>
+     * If not specified, defaults to the platform default encoding.
+     * </p>
+     * 
+     * @param encoding
+     *            Encoding, e.g. "UTF-8".
+     */
+    public void setWebXmlEncoding(String encoding) {
+        webxmlEncoding = encoding;
+    }
+
+    /**
+     * Sets the option to merge generated web.xml fragment into the
+     * WEB-INF/web.xml file of the web application that we were processing.
+     * 
+     * @param b
+     *            <code>true</code> to merge the fragment into the existing
+     *            web.xml file of the processed web application
+     *            ({uriroot}/WEB-INF/web.xml), <code>false</code> to keep the
+     *            generated web.xml fragment
+     */
     public void setAddWebXmlMappings(boolean b) {
         addWebXmlMappings = b;
     }
 
     /**
-     * Set the option that throws an exception in case of a compilation error.
+     * Sets the option that throws an exception in case of a compilation error.
      */
     public void setFailOnError(final boolean b) {
         failOnError = b;
     }
 
+    /**
+     * Returns true if an exception will be thrown in case of a compilation
+     * error.
+     */
     public boolean getFailOnError() {
         return failOnError;
     }
 
     /**
-     * Obtain JSP configuration informantion specified in web.xml.
+     * {@inheritDoc}
      */
     public JspConfig getJspConfig() {
         return jspConfig;
     }
 
+    /**
+     * {@inheritDoc}
+     */
     public TagPluginManager getTagPluginManager() {
         return tagPluginManager;
     }
 
+    /**
+     * Adds servlet declaration and mapping for the JSP page servlet to the
+     * generated web.xml fragment.
+     * 
+     * @param file
+     *            Context-relative path to the JSP file, e.g.
+     *            <code>/index.jsp</code>
+     * @param clctxt
+     *            Compilation context of the servlet
+     */
     public void generateWebMapping( String file, JspCompilationContext clctxt )
         throws IOException
     {
@@ -863,72 +1013,67 @@ public class JspC implements Options {
         String insertEndMarker =
             Localizer.getMessage("jspc.webinc.insertEnd");
 
-        BufferedReader reader = new BufferedReader(new FileReader(webXml));
-        BufferedReader fragmentReader =
-            new BufferedReader(new FileReader(webxmlFile));
-        PrintWriter writer = new PrintWriter(new FileWriter(webXml2));
+        BufferedReader reader = new BufferedReader(openWebxmlReader(webXml));
+        BufferedReader fragmentReader = new BufferedReader(
+                openWebxmlReader(new File(webxmlFile)));
+        PrintWriter writer = new PrintWriter(openWebxmlWriter(webXml2));
 
         // Insert the <servlet> and <servlet-mapping> declarations
-        int pos = -1;
-        String line = null;
-        while (true) {
-            line = reader.readLine();
-            if (line == null) {
-                break;
-            }
-            // Skip anything previously generated by JSPC
-            if (line.indexOf(insertStartMarker) >= 0) {
-                while (true) {
-                    line = reader.readLine();
-                    if (line == null) {
-                        return;
-                    }
-                    if (line.indexOf(insertEndMarker) >= 0) {
-                        line = reader.readLine();
-                        line = reader.readLine();
-                        if (line == null) {
-                            return;
+        boolean inserted = false;
+        int current = reader.read();
+        while (current > -1) {
+            if (current == '<') {
+                String element = getElement(reader);
+                boolean found = false;
+                if (!inserted) {
+                    for (String before : insertBefore) {
+                        if (element.equals(before)) {
+                            found = true;
+                            break;
                         }
-                        break;
                     }
                 }
-            }
-            for (int i = 0; i < insertBefore.length; i++) {
-                pos = line.indexOf(insertBefore[i]);
-                if (pos >= 0)
-                    break;
-            }
-            if (pos >= 0) {
-                writer.print(line.substring(0, pos));
-                break;
+                if (found) {
+                    // Insert generated content here
+                    writer.println(insertStartMarker);
+                    while (true) {
+                        String line = fragmentReader.readLine();
+                        if (line == null) {
+                            writer.println();
+                            break;
+                        }
+                        writer.println(line);
+                    }
+                    writer.println(insertEndMarker);
+                    writer.println();
+                    writer.write(element);
+                    inserted = true;
+                } else if (element.equals(insertStartMarker)) {
+                    // Skip the previous auto-generated content
+                    while (true) {
+                        current = reader.read();
+                        if (current < 0) {
+                            throw new EOFException();
+                        }
+                        if (current == '<') {
+                            element = getElement(reader);
+                            if (element.equals(insertEndMarker)) {
+                                break;
+                            }
+                        }
+                    }
+                    current = reader.read();
+                    while (current == '\n' || current == '\r') {
+                        current = reader.read();
+                    }
+                    continue;
+                } else {
+                    writer.write(element);
+                }
             } else {
-                writer.println(line);
+                writer.write(current);
             }
-        }
-
-        writer.println(insertStartMarker);
-        while (true) {
-            String line2 = fragmentReader.readLine();
-            if (line2 == null) {
-                writer.println();
-                break;
-            }
-            writer.println(line2);
-        }
-        writer.println(insertEndMarker);
-        writer.println();
-
-        for (int i = 0; i < pos; i++) {
-            writer.print(" ");
-        }
-        writer.println(line.substring(pos));
-
-        while (true) {
-            line = reader.readLine();
-            if (line == null) {
-                break;
-            }
-            writer.println(line);
+            current = reader.read();
         }
         writer.close();
 
@@ -955,6 +1100,43 @@ public class JspC implements Options {
 
     }
 
+
+    /*
+     * Assumes valid xml
+     */
+    private String getElement(Reader reader) throws IOException {
+        StringBuilder result = new StringBuilder();
+        result.append('<');
+        
+        boolean done = false;
+        
+        while (!done) {
+            int current = reader.read();
+            while (current != '>') {
+                if (current < 0) {
+                    throw new EOFException();
+                }
+                result.append((char) current);
+                current = reader.read();
+            }
+            result.append((char) current);
+            
+            int len = result.length();
+            if (len > 4 && result.substring(0, 4).equals("<!--")) {
+                // This is a comment - make sure we are at the end
+                if (len >= 7 && result.substring(len - 3, len).equals("-->")) {
+                    done = true;
+                }
+            } else {
+                done = true;
+            }
+        }
+        
+        
+        return result.toString();
+    }
+
+
     protected void processFile(String file)
         throws JasperException
     {
@@ -1208,8 +1390,7 @@ public class JspC implements Options {
     protected void initWebXml() {
         try {
             if (webxmlLevel >= INC_WEBXML) {
-                File fmapings = new File(webxmlFile);
-                mapout = new FileWriter(fmapings);
+                mapout = openWebxmlWriter(new File(webxmlFile));
                 servletout = new CharArrayWriter();
                 mappingout = new CharArrayWriter();
             } else {
@@ -1421,4 +1602,26 @@ public class JspC implements Options {
              return FileUtils.newFileUtils().resolveFile(getProject().getBaseDir(), s);
          }
      }
+
+    private Reader openWebxmlReader(File file) throws IOException {
+        FileInputStream fis = new FileInputStream(file);
+        try {
+            return webxmlEncoding != null ? new InputStreamReader(fis,
+                    webxmlEncoding) : new InputStreamReader(fis);
+        } catch (IOException ex) {
+            fis.close();
+            throw ex;
+        }
+    }
+
+    private Writer openWebxmlWriter(File file) throws IOException {
+        FileOutputStream fos = new FileOutputStream(file);
+        try {
+            return webxmlEncoding != null ? new OutputStreamWriter(fos,
+                    webxmlEncoding) : new OutputStreamWriter(fos);
+        } catch (IOException ex) {
+            fos.close();
+            throw ex;
+        }
+    }
 }
