@@ -39,10 +39,44 @@ import java.util.logging.SimpleFormatter;
 
 /**
  * Implementation of <b>Handler</b> that appends log messages to a file
- * named {prefix}.{date}.{suffix} in a configured directory, with an
- * optional preceding timestamp.
+ * named {prefix}{date}{suffix} in a configured directory.
  *
- * @version $Id: FileHandler.java 939520 2010-04-30 00:14:14Z kkolinko $
+ * <p>The following configuration properties are available:</p>
+ *
+ * <ul>
+ *   <li><code>directory</code> - The directory where to create the log file.
+ *    If the path is not absolute, it is relative to the current working
+ *    directory of the application. The Apache Tomcat configuration files usually
+ *    specify an absolute path for this property,
+ *    <code>${catalina.base}/logs</code> 
+ *    Default value: <code>logs</code></li>
+ *   <li><code>rotatable</code> - If <code>true</code>, the log file will be
+ *    rotated on the first write past midnight and the filename will be
+ *    <code>{prefix}{date}{suffix}</code>, where date is yyyy-MM-dd. If <code>false</code>,
+ *    the file will not be rotated and the filename will be <code>{prefix}{suffix}</code>.
+ *    Default value: <code>true</code></li>
+ *   <li><code>prefix</code> - The leading part of the log file name.
+ *    Default value: <code>juli.</code></li>
+ *   <li><code>suffix</code> - The trailing part of the log file name. Default value: <code>.log</code></li>
+ *   <li><code>bufferSize</code> - Configures buffering. The value of <code>0</code>
+ *    uses system default buffering (typically an 8K buffer will be used). A
+ *    value of <code>&lt;0</code> forces a writer flush upon each log write. A
+ *    value <code>&gt;0</code> uses a BufferedOutputStream with the defined
+ *    value but note that the system default buffering will also be
+ *    applied. Default value: <code>-1</code></li>
+ *   <li><code>encoding</code> - Character set used by the log file. Default value:
+ *    empty string, which means to use the system default character set.</li>
+ *   <li><code>level</code> - The level threshold for this Handler. See the
+ *    <code>java.util.logging.Level</code> class for the possible levels.
+ *    Default value: <code>ALL</code></li>
+ *   <li><code>filter</code> - The <code>java.util.logging.Filter</code>
+ *    implementation class name for this Handler. Default value: unset</li>
+ *   <li><code>formatter</code> - The <code>java.util.logging.Formatter</code>
+ *    implementation class name for this Handler. Default value:
+ *    <code>java.util.logging.SimpleFormatter</code></li>
+ * </ul>
+ *
+ * @version $Id: FileHandler.java 1149099 2011-07-21 10:25:17Z kkolinko $
  */
 
 public class FileHandler
@@ -94,6 +128,12 @@ public class FileHandler
     private String suffix = null;
 
 
+    /**
+     * Determines whether the logfile is rotatable
+     */
+    private boolean rotatable = true;
+
+
     /**
      * The PrintWriter to which we are currently logging, if any.
      */
@@ -133,7 +173,7 @@ public class FileHandler
 
         writerLock.readLock().lock();
         // If the date has changed, switch log files
-        if (!date.equals(tsDate)) {
+        if (rotatable && !date.equals(tsDate)) {
             // Update to writeLock before we switch
             writerLock.readLock().unlock();
             writerLock.writeLock().lock();
@@ -242,6 +282,7 @@ public class FileHandler
         ClassLoader cl = Thread.currentThread().getContextClassLoader();
         
         // Retrieve configuration of logging file name
+        rotatable = Boolean.parseBoolean(getProperty(className + ".rotatable", "true"));
         if (directory == null)
             directory = getProperty(className + ".directory", "logs");
         if (prefix == null)
@@ -283,7 +324,8 @@ public class FileHandler
             try {
                 setFormatter((Formatter) cl.loadClass(formatterName).newInstance());
             } catch (Exception e) {
-                // Ignore
+                // Ignore and fallback to defaults
+                setFormatter(new SimpleFormatter());
             }
         } else {
             setFormatter(new SimpleFormatter());
@@ -322,8 +364,12 @@ public class FileHandler
         // Open the current log file
         writerLock.writeLock().lock();
         try {
-            String pathname = dir.getAbsolutePath() + File.separator +
-                prefix + date + suffix;
+            File pathname = new File(dir.getAbsoluteFile(), prefix
+                    + (rotatable ? date : "") + suffix);
+            File parent = pathname.getParentFile();
+            if (!parent.exists()) {
+                parent.mkdirs();
+            }
             String encoding = getEncoding();
             FileOutputStream fos = new FileOutputStream(pathname, true);
             OutputStream os = bufferSize>0?new BufferedOutputStream(fos,bufferSize):fos;
