@@ -64,7 +64,7 @@ import java.util.logging.LogManager;
  * @author Craig R. McClanahan
  * @author Costin Manolache
  * @author Richard A. Sitze
- * @version $Id: LogFactory.java 939520 2010-04-30 00:14:14Z kkolinko $
+ * @version $Id: LogFactory.java 1024318 2010-10-19 16:44:26Z markt $
  */
 public /* abstract */ class LogFactory {
 
@@ -328,8 +328,13 @@ public /* abstract */ class LogFactory {
      */
     public static void release(
             @SuppressWarnings("unused") ClassLoader classLoader) {
-        // JULI's log manager looks at the current classLoader
-        LogManager.getLogManager().reset();
+        // JULI's log manager looks at the current classLoader, the default
+        // implementation does not so calling reset in that case will break
+        // things
+        if (!LogManager.getLogManager().getClass().getName().equals(
+                "java.util.logging.LogManager")) {
+            LogManager.getLogManager().reset();
+        }
     }
 
 
