@@ -20,7 +20,7 @@ package org.apache.tomcat.jni;
 /** Error
  *
  * @author Mladen Turk
- * @version $Id: Error.java 939351 2010-04-29 15:41:54Z kkolinko $
+ * @version $Id: Error.java 1036981 2010-11-19 19:01:34Z markt $
  */
 
 public class Error extends Exception {
@@ -43,7 +43,7 @@ public class Error extends Exception {
      */
     private Error(int error, String description)
     {
-        super(description);
+        super(error + ": " + description);
         this.error = error;
         this.description = description;
     }
