@@ -157,7 +157,9 @@ public class AprEndpoint {
      */
     protected long sslContext = 0;
 
-    
+    /* Acceptor thread array */
+    private Acceptor acceptors[] = null;
+
     // ------------------------------------------------------------- Properties
 
 
@@ -167,7 +169,7 @@ public class AprEndpoint {
     protected boolean deferAccept = true;
     public void setDeferAccept(boolean deferAccept) { this.deferAccept = deferAccept; }
     public boolean getDeferAccept() { return deferAccept; }
-    
+
 
     /**
      * External Executor based thread pool.
@@ -409,7 +411,7 @@ public class AprEndpoint {
         this.unlockTimeout = unlockTimeout;
     }
 
-    
+
     /**
      * SSL engine.
      */
@@ -564,7 +566,7 @@ public class AprEndpoint {
     }
 
     /**
-     * Return the amount of threads that are in use 
+     * Return the amount of threads that are in use
      *
      * @return the amount of threads that are in use
      */
@@ -608,7 +610,7 @@ public class AprEndpoint {
 
         if (initialized)
             return;
-        
+
         // Create the root APR memory pool
         rootPool = Pool.create(0);
         // Create the pool for the server socket
@@ -689,7 +691,7 @@ public class AprEndpoint {
                 sendfileThreadCount = 1;
             }
         }
-        
+
         // Delay accepting of new connections until data is available
         // Only Linux kernels 2.4 + have that implemented
         // on other platforms this call is noop and will return APR_ENOTIMPL.
@@ -701,7 +703,7 @@ public class AprEndpoint {
 
         // Initialize SSL if needed
         if (SSLEnabled) {
-            
+
             // SSL protocol
             int value = SSL.SSL_PROTOCOL_ALL;
             if ("SSLv2".equalsIgnoreCase(SSLProtocol)) {
@@ -712,7 +714,17 @@ public class AprEndpoint {
                 value = SSL.SSL_PROTOCOL_TLSV1;
             } else if ("SSLv2+SSLv3".equalsIgnoreCase(SSLProtocol)) {
                 value = SSL.SSL_PROTOCOL_SSLV2 | SSL.SSL_PROTOCOL_SSLV3;
+            } else if ("all".equalsIgnoreCase(SSLProtocol) ||
+                    SSLProtocol == null || SSLProtocol.length() == 0) {
+                // NOOP, use the default defined above
+            } else {
+                // Protocol not recognized, fail to start as it is safer than
+                // continuing with the default which might enable more than the
+                // is required
+                throw new Exception(sm.getString(
+                        "endpoint.apr.invalidSslProtocol", SSLProtocol));
             }
+
             // Create SSL Context
             sslContext = SSLContext.make(rootPool, value, SSL.SSL_MODE_SERVER);
             // List the ciphers that the client is permitted to negotiate
@@ -767,10 +779,10 @@ public class AprEndpoint {
             for (int i = 0; i < pollerThreadCount; i++) {
                 pollers[i] = new Poller(false);
                 pollers[i].init();
-                Thread pollerThread = new Thread(pollers[i], getName() + "-Poller-" + i);
-                pollerThread.setPriority(threadPriority);
-                pollerThread.setDaemon(true);
-                pollerThread.start();
+                pollers[i].setName(getName() + "-Poller-" + i);
+                pollers[i].setPriority(threadPriority);
+                pollers[i].setDaemon(true);
+                pollers[i].start();
             }
 
             // Start comet poller threads
@@ -778,10 +790,10 @@ public class AprEndpoint {
             for (int i = 0; i < pollerThreadCount; i++) {
                 cometPollers[i] = new Poller(true);
                 cometPollers[i].init();
-                Thread pollerThread = new Thread(cometPollers[i], getName() + "-CometPoller-" + i);
-                pollerThread.setPriority(threadPriority);
-                pollerThread.setDaemon(true);
-                pollerThread.start();
+                cometPollers[i].setName(getName() + "-CometPoller-" + i);
+                cometPollers[i].setPriority(threadPriority);
+                cometPollers[i].setDaemon(true);
+                cometPollers[i].start();
             }
 
             // Start sendfile threads
@@ -790,19 +802,21 @@ public class AprEndpoint {
                 for (int i = 0; i < sendfileThreadCount; i++) {
                     sendfiles[i] = new Sendfile();
                     sendfiles[i].init();
-                    Thread sendfileThread = new Thread(sendfiles[i], getName() + "-Sendfile-" + i);
-                    sendfileThread.setPriority(threadPriority);
-                    sendfileThread.setDaemon(true);
-                    sendfileThread.start();
+                    sendfiles[i].setName(getName() + "-Sendfile-" + i);
+                    sendfiles[i].setPriority(threadPriority);
+                    sendfiles[i].setDaemon(true);
+                    sendfiles[i].start();
                 }
             }
 
             // Start acceptor threads
+            acceptors = new Acceptor[acceptorThreadCount];
             for (int i = 0; i < acceptorThreadCount; i++) {
-                Thread acceptorThread = new Thread(new Acceptor(), getName() + "-Acceptor-" + i);
-                acceptorThread.setPriority(threadPriority);
-                acceptorThread.setDaemon(daemon);
-                acceptorThread.start();
+                acceptors[i] = new Acceptor();
+                acceptors[i].setName(getName() + "-Acceptor-" + i);
+                acceptors[i].setPriority(threadPriority);
+                acceptors[i].setDaemon(getDaemon());
+                acceptors[i].start();
             }
 
         }
@@ -835,20 +849,56 @@ public class AprEndpoint {
      * Stop the endpoint. This will cause all processing threads to stop.
      */
     public void stop() {
+        if (!paused) {
+            pause();
+        }
         if (running) {
             running = false;
             unlockAccept();
+            for (int i = 0; i < acceptors.length; i++) {
+                long s = System.currentTimeMillis() + 10000;
+                while (acceptors[i].isAlive() && serverSock != 0) {
+                    try {
+                        acceptors[i].interrupt();
+                        acceptors[i].join(1000);
+                    } catch (InterruptedException e) {
+                        // Ignore
+                    }
+                    if (System.currentTimeMillis() >= s) {
+                        log.warn(sm.getString("endpoint.warn.unlockAcceptorFailed",
+                                 acceptors[i].getName()));
+                        // If the Acceptor is still running force
+                        // the hard socket close.
+                        if (serverSock != 0) {
+                            Socket.shutdown(serverSock, Socket.APR_SHUTDOWN_READ);
+                            serverSock = 0;
+                        }
+                    }
+                }
+            }
             for (int i = 0; i < pollers.length; i++) {
-                pollers[i].destroy();
+                try {
+                    pollers[i].destroy();
+                } catch (Exception e) {
+                    // Ignore
+                }
             }
             pollers = null;
             for (int i = 0; i < cometPollers.length; i++) {
-                cometPollers[i].destroy();
+                try {
+                    cometPollers[i].destroy();
+                } catch (Exception e) {
+                    // Ignore
+                }
             }
             cometPollers = null;
             if (useSendfile) {
                 for (int i = 0; i < sendfiles.length; i++) {
-                    sendfiles[i].destroy();
+                    try {
+                        sendfiles[i].destroy();
+                    } catch (Exception e) {
+                        // Ignore
+                    }
                 }
                 sendfiles = null;
             }
@@ -863,15 +913,26 @@ public class AprEndpoint {
         if (running) {
             stop();
         }
-        Pool.destroy(serverSockPool);
-        serverSockPool = 0;
-        // Close server socket
-        Socket.close(serverSock);
-        serverSock = 0;
+        // Destroy pool if it was initialised
+        if (serverSockPool != 0) {
+            Pool.destroy(serverSockPool);
+            serverSockPool = 0;
+        }
+
+        // Close server socket if it was initialised
+        if (serverSock != 0) {
+            Socket.close(serverSock);
+            serverSock = 0;
+        }
+
         sslContext = 0;
-        // Close all APR memory pools and resources
-        Pool.destroy(rootPool);
-        rootPool = 0;
+
+        // Close all APR memory pools and resources if initialised
+        if (rootPool != 0) {
+            Pool.destroy(rootPool);
+            rootPool = 0;
+        }
+
         initialized = false;
     }
 
@@ -901,7 +962,7 @@ public class AprEndpoint {
                 saddr = new InetSocketAddress(address,port);
             }
             s = new java.net.Socket();
-            s.setSoTimeout(soTimeout);
+            s.setSoTimeout(soTimeout > 0 ? soTimeout : 60000);
             s.setSoLinger(true ,0);
             if (log.isDebugEnabled()) {
                 log.debug("About to unlock socket for: " + saddr);
@@ -1058,7 +1119,7 @@ public class AprEndpoint {
         }
     }
 
-    
+
     /**
      * Allocate a new poller of the specified size.
      */
@@ -1076,7 +1137,7 @@ public class AprEndpoint {
         }
     }
 
-    
+
     /**
      * Process given socket.
      */
@@ -1095,7 +1156,7 @@ public class AprEndpoint {
         }
         return true;
     }
-    
+
 
     /**
      * Process given socket.
@@ -1115,7 +1176,7 @@ public class AprEndpoint {
         }
         return true;
     }
-    
+
 
     /**
      * Process given socket for an event.
@@ -1135,7 +1196,16 @@ public class AprEndpoint {
         }
         return true;
     }
-    
+
+    private void destroySocket(long socket) {
+        if (running && socket != 0) {
+            // If not running the socket will be destroyed by
+            // parent pool or acceptor socket.
+            // In any case disable double free which would cause JVM core.
+            Socket.destroy(socket);
+        }
+    }
+
 
     // --------------------------------------------------- Acceptor Inner Class
 
@@ -1143,8 +1213,9 @@ public class AprEndpoint {
     /**
      * Server socket acceptor thread.
      */
-    protected class Acceptor implements Runnable {
+    protected class Acceptor extends Thread {
 
+        private final Log log = LogFactory.getLog(AprEndpoint.Acceptor.class);
 
         /**
          * The background thread that listens for incoming TCP/IP connections and
@@ -1156,7 +1227,7 @@ public class AprEndpoint {
             while (running) {
 
                 // Loop if endpoint is paused
-                while (paused) {
+                while (paused && running) {
                     try {
                         Thread.sleep(1000);
                     } catch (InterruptedException e) {
@@ -1164,6 +1235,9 @@ public class AprEndpoint {
                     }
                 }
 
+                if (!running) {
+                    break;
+                }
                 try {
                     // Accept the next incoming connection from the server socket
                     long socket = Socket.accept(serverSock);
@@ -1173,16 +1247,31 @@ public class AprEndpoint {
                      * socket and don't process it.
                      */
                     if (deferAccept && (paused || !running)) {
-                        Socket.destroy(socket);
+                        destroySocket(socket);
                         continue;
                     }
                     // Hand this socket off to an appropriate processor
                     if (!processSocketWithOptions(socket)) {
                         // Close socket and pool right away
-                        Socket.destroy(socket);
+                        destroySocket(socket);
                     }
                 } catch (Throwable t) {
-                    if (running) log.error(sm.getString("endpoint.accept.fail"), t);
+                    if (running) {
+                        String msg = sm.getString("endpoint.accept.fail");
+                        if (t instanceof Error) {
+                            Error e = (Error) t;
+                            if (e.getError() == 233) {
+                                // Not an error on HP-UX so log as a warning
+                                // so it can be filtered out on that platform
+                                // See bug 50273
+                                log.warn(msg, t);
+                            } else {
+                                log.error(msg, t);
+                            }
+                        } else {
+                                log.error(msg, t);
+                        }
+                    }
                 }
 
                 // The processor will recycle itself when it finishes
@@ -1200,7 +1289,7 @@ public class AprEndpoint {
     /**
      * Poller class.
      */
-    public class Poller implements Runnable {
+    public class Poller extends Thread {
 
         protected long serverPollset = 0;
         protected long pool = 0;
@@ -1208,7 +1297,7 @@ public class AprEndpoint {
 
         protected long[] addS;
         protected volatile int addCount = 0;
-        
+
         protected boolean comet = true;
 
         protected volatile int keepAliveCount = 0;
@@ -1217,7 +1306,7 @@ public class AprEndpoint {
         public Poller(boolean comet) {
             this.comet = comet;
         }
-        
+
         /**
          * Create the poller. With some versions of APR, the maximum poller size will
          * be 62 (recompiling APR is necessary to remove this limitation).
@@ -1247,23 +1336,13 @@ public class AprEndpoint {
         /**
          * Destroy the poller.
          */
-        protected void destroy() {
-            // Wait for polltime before doing anything, so that the poller threads
-            // exit, otherwise parallel descturction of sockets which are still
-            // in the poller can cause problems
-            try {
-                synchronized (this) {
-                    this.wait(pollTime / 1000);
-                }
-            } catch (InterruptedException e) {
-                // Ignore
-            }
+        public void destroy() {
             // Close all sockets in the add queue
             for (int i = 0; i < addCount; i++) {
                 if (comet) {
                     processSocket(addS[i], SocketStatus.STOP);
                 } else {
-                    Socket.destroy(addS[i]);
+                    destroySocket(addS[i]);
                 }
             }
             // Close all sockets still in the poller
@@ -1273,13 +1352,21 @@ public class AprEndpoint {
                     if (comet) {
                         processSocket(desc[n*2+1], SocketStatus.STOP);
                     } else {
-                        Socket.destroy(desc[n*2+1]);
+                        destroySocket(desc[n*2+1]);
                     }
                 }
             }
             Pool.destroy(pool);
             keepAliveCount = 0;
             addCount = 0;
+            try {
+                while (this.isAlive()) {
+                    this.interrupt();
+                    this.join(1000);
+                }
+            } catch (InterruptedException e) {
+                // Ignore
+            }
         }
 
         /**
@@ -1299,7 +1386,7 @@ public class AprEndpoint {
                     if (comet) {
                         processSocket(socket, SocketStatus.ERROR);
                     } else {
-                        Socket.destroy(socket);
+                        destroySocket(socket);
                     }
                     return;
                 }
@@ -1319,7 +1406,7 @@ public class AprEndpoint {
             // Loop until we receive a shutdown command
             while (running) {
                 // Loop if endpoint is paused
-                while (paused) {
+                while (paused && running) {
                     try {
                         Thread.sleep(1000);
                     } catch (InterruptedException e) {
@@ -1327,9 +1414,12 @@ public class AprEndpoint {
                     }
                 }
 
+                if (!running) {
+                    break;
+                }
                 if (keepAliveCount < 1 && addCount < 1) {
                     synchronized (this) {
-                        while (keepAliveCount < 1 && addCount < 1) {
+                        while (keepAliveCount < 1 && addCount < 1 && running) {
                             // Reset maintain time.
                             maintainTime = 0;
                             try {
@@ -1341,6 +1431,9 @@ public class AprEndpoint {
                     }
                 }
 
+                if (!running) {
+                    break;
+                }
                 try {
                     // Add sockets which are waiting to the poller
                     if (addCount > 0) {
@@ -1357,7 +1450,7 @@ public class AprEndpoint {
                                         if (comet) {
                                             processSocket(addS[i], SocketStatus.ERROR);
                                         } else {
-                                            Socket.destroy(addS[i]);
+                                            destroySocket(addS[i]);
                                         }
                                     }
                                 }
@@ -1377,13 +1470,13 @@ public class AprEndpoint {
                             // Check for failed sockets and hand this socket off to a worker
                             if (((desc[n*2] & Poll.APR_POLLHUP) == Poll.APR_POLLHUP)
                                     || ((desc[n*2] & Poll.APR_POLLERR) == Poll.APR_POLLERR)
-                                    || (comet && (!processSocket(desc[n*2+1], SocketStatus.OPEN))) 
+                                    || (comet && (!processSocket(desc[n*2+1], SocketStatus.OPEN)))
                                     || (!comet && (!processSocket(desc[n*2+1])))) {
                                 // Close socket and clear pool
                                 if (comet) {
                                     processSocket(desc[n*2+1], SocketStatus.DISCONNECT);
                                 } else {
-                                    Socket.destroy(desc[n*2+1]);
+                                    destroySocket(desc[n*2+1]);
                                 }
                                 continue;
                             }
@@ -1414,7 +1507,7 @@ public class AprEndpoint {
                                 if (comet) {
                                     processSocket(desc[n], SocketStatus.TIMEOUT);
                                 } else {
-                                    Socket.destroy(desc[n]);
+                                    destroySocket(desc[n]);
                                 }
                             }
                         }
@@ -1428,9 +1521,7 @@ public class AprEndpoint {
             synchronized (this) {
                 this.notifyAll();
             }
-
         }
-        
     }
 
 
@@ -1571,7 +1662,7 @@ public class AprEndpoint {
                         getPoller().add(socket);
                     } else {
                         // Close socket and pool
-                        Socket.destroy(socket);
+                        destroySocket(socket);
                         socket = 0;
                     }
                 } else {
@@ -1579,12 +1670,12 @@ public class AprEndpoint {
                     // Process the request from this socket
                     if ((status != null) && (handler.event(socket, status) == Handler.SocketState.CLOSED)) {
                         // Close socket and pool
-                        Socket.destroy(socket);
+                        destroySocket(socket);
                         socket = 0;
-                    } else if ((status == null) && ((options && !setSocketOptions(socket)) 
+                    } else if ((status == null) && ((options && !setSocketOptions(socket))
                             || handler.process(socket) == Handler.SocketState.CLOSED)) {
                         // Close socket and pool
-                        Socket.destroy(socket);
+                        destroySocket(socket);
                         socket = 0;
                     }
                 }
@@ -1640,13 +1731,13 @@ public class AprEndpoint {
     /**
      * Sendfile class.
      */
-    public class Sendfile implements Runnable {
+    public class Sendfile extends Thread {
 
         protected long sendfilePollset = 0;
         protected long pool = 0;
         protected long[] desc;
         protected HashMap<Long, SendfileData> sendfileData;
-        
+
         protected volatile int sendfileCount;
         public int getSendfileCount() { return sendfileCount; }
 
@@ -1678,32 +1769,31 @@ public class AprEndpoint {
         /**
          * Destroy the poller.
          */
-        protected void destroy() {
-            // Wait for polltime before doing anything, so that the poller threads
-            // exit, otherwise parallel descturction of sockets which are still
-            // in the poller can cause problems
-            try {
-                synchronized (this) {
-                    this.wait(pollTime / 1000);
-                }
-            } catch (InterruptedException e) {
-                // Ignore
-            }
+        public void destroy() {
             // Close any socket remaining in the add queue
             addCount = 0;
             for (int i = (addS.size() - 1); i >= 0; i--) {
                 SendfileData data = addS.get(i);
-                Socket.destroy(data.socket);
+                destroySocket(data.socket);
             }
+            addS.clear();
             // Close all sockets still in the poller
             int rv = Poll.pollset(sendfilePollset, desc);
             if (rv > 0) {
                 for (int n = 0; n < rv; n++) {
-                    Socket.destroy(desc[n*2+1]);
+                    destroySocket(desc[n*2+1]);
                 }
             }
             Pool.destroy(pool);
             sendfileData.clear();
+            try {
+                while (this.isAlive()) {
+                    this.interrupt();
+                    this.join(1000);
+                }
+            } catch (InterruptedException e) {
+                // Ignore
+            }
         }
 
         /**
@@ -1732,7 +1822,9 @@ public class AprEndpoint {
                                                data.pos, data.end - data.pos, 0);
                     if (nw < 0) {
                         if (!(-nw == Status.EAGAIN)) {
-                            Socket.destroy(data.socket);
+                            Pool.destroy(data.fdpool);
+                            // No need to close socket, this will be done by
+                            // calling code since data.socket == 0
                             data.socket = 0;
                             return false;
                         } else {
@@ -1788,7 +1880,7 @@ public class AprEndpoint {
             while (running) {
 
                 // Loop if endpoint is paused
-                while (paused) {
+                while (paused && running) {
                     try {
                         Thread.sleep(1000);
                     } catch (InterruptedException e) {
@@ -1796,9 +1888,12 @@ public class AprEndpoint {
                     }
                 }
 
+                if (!running) {
+                    break;
+                }
                 if (sendfileCount < 1 && addCount < 1) {
                     synchronized (this) {
-                        while (sendfileCount < 1 && addS.size() < 1) {
+                        while (sendfileCount < 1 && addS.size() < 1 && running) {
                             // Reset maintain time.
                             maintainTime = 0;
                             try {
@@ -1810,6 +1905,9 @@ public class AprEndpoint {
                     }
                 }
 
+                if (!running) {
+                    break;
+                }
                 try {
                     // Add socket to the poller
                     if (addCount > 0) {
@@ -1825,7 +1923,7 @@ public class AprEndpoint {
                                     } else {
                                         log.warn(sm.getString("endpoint.sendfile.addfail", "" + rv, Error.strerror(rv)));
                                         // Can't do anything: close the socket right away
-                                        Socket.destroy(data.socket);
+                                        destroySocket(data.socket);
                                     }
                                 }
                             } finally {
@@ -1851,7 +1949,7 @@ public class AprEndpoint {
                                 remove(state);
                                 // Destroy file descriptor pool, which should close the file
                                 // Close the socket, as the reponse would be incomplete
-                                Socket.destroy(state.socket);
+                                destroySocket(state.socket);
                                 continue;
                             }
                             // Write some data using sendfile
@@ -1863,7 +1961,7 @@ public class AprEndpoint {
                                 remove(state);
                                 // Close the socket, as the reponse would be incomplete
                                 // This will close the file too.
-                                Socket.destroy(state.socket);
+                                destroySocket(state.socket);
                                 continue;
                             }
 
@@ -1880,7 +1978,7 @@ public class AprEndpoint {
                                 } else {
                                     // Close the socket since this is
                                     // the end of not keep-alive request.
-                                    Socket.destroy(state.socket);
+                                    destroySocket(state.socket);
                                 }
                             }
                         }
@@ -1912,7 +2010,7 @@ public class AprEndpoint {
                                 remove(state);
                                 // Destroy file descriptor pool, which should close the file
                                 // Close the socket, as the response would be incomplete
-                                Socket.destroy(state.socket);
+                                destroySocket(state.socket);
                             }
                         }
                     }
@@ -1951,18 +2049,18 @@ public class AprEndpoint {
 
 
     public class WorkerStack {
-        
+
         protected Worker[] workers = null;
         protected int end = 0;
-        
+
         public WorkerStack(int size) {
             workers = new Worker[size];
         }
-        
-        /** 
+
+        /**
          * Put the object into the queue. If the queue is full (for example if
          * the queue has been reduced in size) the object will be dropped.
-         * 
+         *
          * @param   object  the object to be appended to the queue (first
          *                  element).
          */
@@ -1973,10 +2071,10 @@ public class AprEndpoint {
                 curThreads--;
             }
         }
-        
+
         /**
          * Get the first object out of the queue. Return null if the queue
-         * is empty. 
+         * is empty.
          */
         public Worker pop() {
             if (end > 0) {
@@ -1984,7 +2082,7 @@ public class AprEndpoint {
             }
             return null;
         }
-        
+
         /**
          * Get the first object out of the queue, Return null if the queue
          * is empty.
@@ -1992,25 +2090,25 @@ public class AprEndpoint {
         public Worker peek() {
             return workers[end];
         }
-        
+
         /**
          * Is the queue empty?
          */
         public boolean isEmpty() {
             return (end == 0);
         }
-        
+
         /**
          * How many elements are there in this queue?
          */
         public int size() {
             return (end);
         }
-        
+
         /**
          * Resize the queue. If there are too many objects in the queue for the
          * new size, drop the excess.
-         * 
+         *
          * @param newSize
          */
         public void resize(int newSize) {
@@ -2034,9 +2132,9 @@ public class AprEndpoint {
      * and do the handshake.
      */
     protected class SocketWithOptionsProcessor implements Runnable {
-        
+
         protected long socket = 0;
-        
+
         public SocketWithOptionsProcessor(long socket) {
             this.socket = socket;
         }
@@ -2048,24 +2146,24 @@ public class AprEndpoint {
                     getPoller().add(socket);
                 } else {
                     // Close socket and pool
-                    Socket.destroy(socket);
+                    destroySocket(socket);
                     socket = 0;
                 }
             } else {
                 // Process the request from this socket
-                if (!setSocketOptions(socket) 
+                if (!setSocketOptions(socket)
                         || handler.process(socket) == Handler.SocketState.CLOSED) {
                     // Close socket and pool
-                    Socket.destroy(socket);
+                    destroySocket(socket);
                     socket = 0;
                 }
             }
 
         }
-        
+
     }
-    
-    
+
+
     // ---------------------------------------------- SocketProcessor Inner Class
 
 
@@ -2074,9 +2172,9 @@ public class AprEndpoint {
      * external Executor thread pool.
      */
     protected class SocketProcessor implements Runnable {
-        
+
         protected long socket = 0;
-        
+
         public SocketProcessor(long socket) {
             this.socket = socket;
         }
@@ -2086,15 +2184,15 @@ public class AprEndpoint {
             // Process the request from this socket
             if (handler.process(socket) == Handler.SocketState.CLOSED) {
                 // Close socket and pool
-                Socket.destroy(socket);
+                destroySocket(socket);
                 socket = 0;
             }
 
         }
-        
+
     }
-    
-    
+
+
     // --------------------------------------- SocketEventProcessor Inner Class
 
 
@@ -2103,10 +2201,10 @@ public class AprEndpoint {
      * external Executor thread pool.
      */
     protected class SocketEventProcessor implements Runnable {
-        
+
         protected long socket = 0;
-        protected SocketStatus status = null; 
-        
+        protected SocketStatus status = null;
+
         public SocketEventProcessor(long socket, SocketStatus status) {
             this.socket = socket;
             this.status = status;
@@ -2117,13 +2215,13 @@ public class AprEndpoint {
             // Process the request from this socket
             if (handler.event(socket, status) == Handler.SocketState.CLOSED) {
                 // Close socket and pool
-                Socket.destroy(socket);
+                destroySocket(socket);
                 socket = 0;
             }
 
         }
-        
+
     }
-    
-    
+
+
 }
